@@ -4,7 +4,7 @@
  * radiusd.h	Structures, prototypes and global variables
  *		for the FreeRADIUS server.
  *
- * Version:	$Id: radiusd.h,v 1.121 2002/11/12 20:22:46 aland Exp $
+ * Version:	$Id: radiusd.h,v 1.139 2003/07/04 19:11:07 aland Exp $
  *
  */
 #include "libradius.h"
@@ -70,6 +70,7 @@ typedef struct auth_req {
 #define RAD_REQUEST_OPTION_NONE            (0)
 #define RAD_REQUEST_OPTION_LOGGED_CHILD    (1 << 0)
 #define RAD_REQUEST_OPTION_DELAYED_REJECT  (1 << 1)
+#define RAD_REQUEST_OPTION_DONT_CACHE      (1 << 2)
 
 /*
  *  Function handler for requests.
@@ -99,22 +100,21 @@ typedef struct nas {
 typedef struct _realm {
 	char			realm[64];
 	char			server[64];
-	uint32_t		ipaddr;
+	char			acct_server[64];
+	uint32_t		ipaddr;	/* authentication */
 	uint32_t		acct_ipaddr;
 	u_char			secret[32];
+	time_t			last_reply; /* last time we saw a packet */
 	int			auth_port;
 	int			acct_port;
 	int			striprealm;
-	int			trusted;
+	int			trusted; /* old */
 	int			notrealm;
-	int			active;
-	time_t			wakeup;
+	int			active;	/* is it dead? */
+	time_t			wakeup;	/* when we should try it again */
 	int			acct_active;
 	time_t			acct_wakeup;
 	int			ldflag;
-	int			chose;
-	int			node;
-	int			total;
 	struct _realm		*next;
 } REALM;
 
@@ -135,15 +135,19 @@ typedef struct main_config_t {
 	int		log_auth_badpass;
 	int		log_auth_goodpass;
 	int		do_usercollide;
+#ifdef WITH_SNMP
+	int		do_snmp;
+#endif
 	int		allow_core_dumps;
 	int		debug_level;
 	int		proxy_requests;
+	int		post_proxy_authorize;
+	int		wake_all_if_all_dead;
 	int		proxy_synchronous;
 	int		proxy_dead_time;
 	int		proxy_retry_count;
 	int		proxy_retry_delay;
 	int		proxy_fallback;
-	int		max_proxies;
 	int		reject_delay;
 	int		status_server;
 	int		max_request_time;
@@ -175,7 +179,6 @@ typedef struct main_config_t {
 #define RETRY_DELAY             5
 #define RETRY_COUNT             3
 #define DEAD_TIME               120
-#define MAX_PROXIES		15
 
 #define L_DBG			1
 #define L_AUTH			2
@@ -203,7 +206,8 @@ typedef enum radlog_dest_t {
   RADLOG_FILES = 0,
   RADLOG_SYSLOG,
   RADLOG_STDOUT,
-  RADLOG_STDERR
+  RADLOG_STDERR,
+  RADLOG_NULL
 } radlog_dest_t;
 
 /*
@@ -235,17 +239,15 @@ extern const char      *radiusd_version;
 int		rad_accounting(REQUEST *);
 
 /* session.c */
-int		rad_check_ts(uint32_t nasaddr, int port, const char *user,
+int		rad_check_ts(uint32_t nasaddr, unsigned int port, const char *user,
 			     const char *sessionid);
-int		session_zap(int fd, uint32_t nasaddr, int port, const char *user,
+int		session_zap(int fd, uint32_t nasaddr, unsigned int port, const char *user,
 			    const char *sessionid, uint32_t cliaddr,
 			    char proto, time_t t);
 
 /* radiusd.c */
 void		debug_pair(FILE *, VALUE_PAIR *);
 int		log_err (char *);
-void		sig_cleanup(int);
-void		queue_sig_cleanup(int);
 int		rad_process(REQUEST *, int);
 int		rad_respond(REQUEST *, RAD_REQUEST_FUNP fun);
 
@@ -274,7 +276,6 @@ int		pairlist_read(const char *file, PAIR_LIST **list, int complain);
 void		pairlist_free(PAIR_LIST **);
 int		read_config_files(void);
 int		read_realms_file(const char *file);
-extern		void check_proxies(int);
 
 /* nas.c */
 int		read_naslist_file(char *);
@@ -308,13 +309,13 @@ int proxy_send(REQUEST *request);
 /* auth.c */
 char	*auth_name(char *buf, size_t buflen, REQUEST *request, int do_cli);
 int		rad_authenticate (REQUEST *);
-VALUE_PAIR	*rad_getpass(REQUEST *request);
-int             rad_check_return(VALUE_PAIR *list);
 int		rad_check_password(REQUEST *request);
 
 /* exec.c */
-int		radius_exec_program(const char *,  REQUEST *,
-				    int, const char **user_msg);
+int		radius_exec_program(const char *,  REQUEST *, int,
+				    char *user_msg, int msg_len,
+				    VALUE_PAIR *input_pairs,
+				    VALUE_PAIR **output_pairs);
 
 /* timestr.c */
 int		timestr_match(char *, time_t);
@@ -347,6 +348,7 @@ extern		void rad_exec_init(void);
 extern		pid_t rad_fork(int exec_wait);
 extern		pid_t rad_waitpid(pid_t pid, int *status, int options);
 extern		int rad_savepid(pid_t pid, int status);
+extern          int total_active_threads(void);
 
 #ifndef HAVE_PTHREAD_H
 #define rad_fork(n) fork()
