@@ -1,11 +1,11 @@
 /*
  * radius.c	Functions to send/receive radius packets.
  *
- * Version:	$Id: radius.c,v 1.86 2002/10/25 16:30:56 aland Exp $
+ * Version:	$Id: radius.c,v 1.99.2.1 2003/07/15 14:19:25 aland Exp $
  *
  */
 
-static const char rcsid[] = "$Id: radius.c,v 1.86 2002/10/25 16:30:56 aland Exp $";
+static const char rcsid[] = "$Id: radius.c,v 1.99.2.1 2003/07/15 14:19:25 aland Exp $";
 
 #include	"autoconf.h"
 #include	"md5.h"
@@ -62,11 +62,12 @@ typedef struct radius_packet_t {
   uint8_t	code;
   uint8_t	id;
   uint8_t	length[2];
-  uint8_t	vector[16];
+  uint8_t	vector[AUTH_VECTOR_LEN];
   uint8_t	data[1];
 } radius_packet_t;
 
-static uint8_t random_vector_pool[AUTH_VECTOR_LEN*2];
+static lrad_randctx lrad_rand_pool;	/* across multiple calls */
+static int lrad_pool_initialized = 0;
 
 static const char *packet_codes[] = {
   "",
@@ -82,7 +83,45 @@ static const char *packet_codes[] = {
   "Accounting-Message",
   "Access-Challenge",
   "Status-Server",
-  "Status-Client"
+  "Status-Client",
+  "14",
+  "15",
+  "16",
+  "17",
+  "18",
+  "19",
+  "20",
+  "Resource-Free-Request",
+  "Resource-Free-Response",
+  "Resource-Query-Request",
+  "Resource-Query-Response",
+  "Alternate-Resource-Reclaim-Request",
+  "NAS-Reboot-Request",
+  "NAS-Reboot-Response",
+  "28",
+  "Next-Passcode",
+  "New-Pin",
+  "Terminate-Session",
+  "Password-Expired",
+  "Event-Request",
+  "Event-Response",
+  "35",
+  "36",
+  "37",
+  "38",
+  "39",
+  "Disconnect-Request",
+  "Disconnect-ACK",
+  "Disconnect-NAK",
+  "CoF-Request",
+  "CoF-ACK",
+  "CoF-NAK",
+  "46",
+  "47",
+  "48",
+  "49",
+  "IP-Address-Allocate",
+  "IP-Address-Release"
 };
 
 /*
@@ -95,7 +134,8 @@ static void make_secret(unsigned char *digest, uint8_t *vector,
  *	Reply to the request.  Also attach
  *	reply attribute value pairs and any user message provided.
  */
-int rad_send(RADIUS_PACKET *packet, const RADIUS_PACKET *original, const char *secret)
+int rad_send(RADIUS_PACKET *packet, const RADIUS_PACKET *original,
+	     const char *secret)
 {
 	VALUE_PAIR		*reply;
 	struct	sockaddr_in	saremote;
@@ -103,7 +143,7 @@ int rad_send(RADIUS_PACKET *packet, const RADIUS_PACKET *original, const char *s
 	const char		*what;
 	uint8_t			ip_buffer[16];
 
-	if ((packet->code > 0) && (packet->code < 14)) {
+	if ((packet->code > 0) && (packet->code < 52)) {
 		what = packet_codes[packet->code];
 	} else {
 		what = "Reply";
@@ -135,7 +175,8 @@ int rad_send(RADIUS_PACKET *packet, const RADIUS_PACKET *original, const char *s
 		   */
 		  hdr->code = packet->code;
 		  hdr->id = packet->id;
-		  if (packet->code == PW_ACCOUNTING_REQUEST) {
+		  if ((packet->code == PW_ACCOUNTING_REQUEST) ||
+		      (packet->code == PW_DISCONNECT_REQUEST)) {
 			  memset(hdr->vector, 0, AUTH_VECTOR_LEN);
 		  } else {
 			  memcpy(hdr->vector, packet->vector, AUTH_VECTOR_LEN);
@@ -177,6 +218,13 @@ int rad_send(RADIUS_PACKET *packet, const RADIUS_PACKET *original, const char *s
 				  msg_auth_offset = total_length;
 			  }
 
+			  /*
+			   *	Print out ONLY the attributes which
+			   *	we're sending over the wire, and print
+			   *	them out BEFORE they're encrypted.
+			   */
+			  debug_pair(reply);
+
 			  /*
 			   *	We have a different vendor.  Re-set
 			   *	the vendor codes.
@@ -299,8 +347,8 @@ int rad_send(RADIUS_PACKET *packet, const RADIUS_PACKET *original, const char *s
 				  case FLAG_ENCRYPT_USER_PASSWORD:
 				    rad_pwencode((char *)reply->strvalue,
 						 &(reply->length),
-						 (char *)secret,
-						 (char *)packet->vector);
+						 (const char *)secret,
+						 (const char *)packet->vector);
 				    break;
 
 				  case FLAG_ENCRYPT_TUNNEL_PASSWORD:
@@ -420,14 +468,6 @@ int rad_send(RADIUS_PACKET *packet, const RADIUS_PACKET *original, const char *s
 			  default:
 				  break;
 			  }
-			  
-			  /*
-			   *	Print out ONLY the attributes which
-			   *	we're sending over the wire.  Also,
-			   *	pick up any hacked password
-			   *	attributes.
-			   */
-			  debug_pair(reply);
 		  } /* done looping over all attributes */
 
 		  /*
@@ -565,7 +605,7 @@ int rad_send(RADIUS_PACKET *packet, const RADIUS_PACKET *original, const char *s
 static int calc_acctdigest(RADIUS_PACKET *packet, const char *secret)
 {
 	u_char		digest[AUTH_VECTOR_LEN];
-	MD5_CTX	context;
+	MD5_CTX		context;
 
 	/*
 	 *	Older clients have the authentication vector set to
@@ -606,15 +646,23 @@ static int calc_acctdigest(RADIUS_PACKET *packet, const char *secret)
  *	Validates the requesting client NAS.  Calculates the
  *	signature based on the clients private key.
  */
-static int calc_replydigest(RADIUS_PACKET *packet, RADIUS_PACKET *original, const char *secret)
+static int calc_replydigest(RADIUS_PACKET *packet, RADIUS_PACKET *original,
+			    const char *secret)
 {
 	uint8_t		calc_digest[AUTH_VECTOR_LEN];
 	MD5_CTX		context;
 
+	/*
+	 *	Very bad!
+	 */
+	if (original == NULL) {
+		return 3;
+	}
+
 	/*
 	 *  Copy the original vector in place.
 	 */
-	memcpy(packet->data + 4, original->vector, sizeof(original->vector));
+	memcpy(packet->data + 4, original->vector, AUTH_VECTOR_LEN);
 
 	/*
 	 *  MD5(packet + secret);
@@ -627,13 +675,13 @@ static int calc_replydigest(RADIUS_PACKET *packet, RADIUS_PACKET *original, cons
 	/*
 	 *  Copy the packet's vector back to the packet.
 	 */
-	memcpy(packet->data + 4, packet->vector, sizeof(packet->vector));
+	memcpy(packet->data + 4, packet->vector, AUTH_VECTOR_LEN);
 
 	/*
 	 *	Return 0 if OK, 2 if not OK.
 	 */
 	packet->verified =
-		memcmp(packet->vector, calc_digest, sizeof(packet->vector)) ? 2 : 0;
+		memcmp(packet->vector, calc_digest, AUTH_VECTOR_LEN) ? 2 : 0;
 	return packet->verified;
 }
 
@@ -694,6 +742,12 @@ RADIUS_PACKET *rad_recv(int fd)
 	packet->src_ipaddr = saremote.sin_addr.s_addr;
 	packet->src_port = ntohs(saremote.sin_port);
 
+	/*
+	 *	FIXME: Do even more filtering by only permitting
+	 *	certain IP's.  The problem is that we don't know
+	 *	how to do this properly for all possible clients...
+	 */
+
 	/*
 	 *	Explicitely set the VP list to empty.
 	 */
@@ -741,13 +795,12 @@ RADIUS_PACKET *rad_recv(int fd)
 	 *	Code of 16 or greate is not understood.
 	 */
 	if ((hdr->code == 0) ||
-	    (hdr->code >= 16)) {
+	    (hdr->code >= 52)) {
 		librad_log("WARNING: Bad RADIUS packet from host %s: unknown packet code %d",
 			   ip_ntoa(host_ipaddr, packet->src_ipaddr),
 			   hdr->code);
 		free(packet);
 		return NULL;
-		
 	}
 
 	/*
@@ -942,18 +995,6 @@ RADIUS_PACKET *rad_recv(int fd)
 		return NULL;
 	}
 
-	/*
-	 *	If we've seen an EAP-Message attribute without a
-	 *	Message-Authenticator attribute, it's invalid.
-	 */
-	if (((seen_eap & PW_EAP_MESSAGE) == PW_EAP_MESSAGE) &&
-	    ((seen_eap & PW_MESSAGE_AUTHENTICATOR) != PW_MESSAGE_AUTHENTICATOR)) {
-		librad_log("WARNING: Malformed RADIUS packet from host %s: Contains EAP-Message, but no Message-Authenticator",
-			   ip_ntoa(host_ipaddr, packet->src_ipaddr));
-		free(packet);
-		return NULL;
-	}
-
 	/*
 	 *	If we're configured to look for a maximum number of
 	 *	attributes, and we've seen more than that maximum,
@@ -969,7 +1010,7 @@ RADIUS_PACKET *rad_recv(int fd)
 	}
 
 	if (librad_debug) {
-		if ((hdr->code > 0) && (hdr->code < 14)) {
+		if ((hdr->code > 0) && (hdr->code < 52)) {
 			printf("rad_recv: %s packet from host %s:%d",
 			       packet_codes[hdr->code],
 			       ip_ntoa(host_ipaddr, packet->src_ipaddr), packet->src_port);
@@ -1006,7 +1047,8 @@ RADIUS_PACKET *rad_recv(int fd)
 /*
  *	Calculate/check digest, and decode radius attributes.
  */
-int rad_decode(RADIUS_PACKET *packet, RADIUS_PACKET *original, const char *secret)
+int rad_decode(RADIUS_PACKET *packet, RADIUS_PACKET *original,
+	       const char *secret)
 {
 	DICT_ATTR		*attr;
 	uint32_t		lvalue;
@@ -1064,7 +1106,7 @@ int rad_decode(RADIUS_PACKET *packet, RADIUS_PACKET *original, const char *secre
 			if (memcmp(calc_auth_vector, msg_auth_vector,
 				    sizeof(calc_auth_vector)) != 0) {
 				char buffer[32];
-				librad_log("Received packet from %s with invalid Message-Authenticator!",
+				librad_log("Received packet from %s with invalid Message-Authenticator!  (Shared secret is incorrect.)",
 					   ip_ntoa(buffer, packet->src_ipaddr));
 				return 1;
 			} /* else the message authenticator was good */
@@ -1085,8 +1127,11 @@ int rad_decode(RADIUS_PACKET *packet, RADIUS_PACKET *original, const char *secre
 	 *	Calculate and/or verify digest.
 	 */
 	switch(packet->code) {
+		int rcode;
+
 		case PW_AUTHENTICATION_REQUEST:
 		case PW_STATUS_SERVER:
+		case PW_DISCONNECT_REQUEST:
 			/*
 			 *	The authentication vector is random
 			 *	nonsense, invented by the client.
@@ -1097,7 +1142,7 @@ int rad_decode(RADIUS_PACKET *packet, RADIUS_PACKET *original, const char *secre
 			if (calc_acctdigest(packet, secret) > 1) {
 				char buffer[32];
 				librad_log("Received Accounting-Request packet "
-				    "from %s with invalid signature!",
+				    "from %s with invalid signature!  (Shared secret is incorrect.)",
 				    ip_ntoa(buffer, packet->src_ipaddr));
 				return 1;
 			}
@@ -1107,12 +1152,14 @@ int rad_decode(RADIUS_PACKET *packet, RADIUS_PACKET *original, const char *secre
 		case PW_AUTHENTICATION_ACK:
 		case PW_AUTHENTICATION_REJECT:
 		case PW_ACCOUNTING_RESPONSE:
-			if (calc_replydigest(packet, original, secret) > 1) {
+			rcode = calc_replydigest(packet, original, secret);
+			if (rcode > 1) {
 				char buffer[32];
 				librad_log("Received %s packet "
-					   "from %s with invalid signature!",
+					   "from %s with invalid signature (err=%d)!  (Shared secret is incorrect.)",
 					   packet_codes[packet->code],
-					   ip_ntoa(buffer, packet->src_ipaddr));
+					   ip_ntoa(buffer, packet->src_ipaddr),
+					   rcode);
 				return 1;
 			}
 		  break;
@@ -1259,16 +1306,18 @@ int rad_decode(RADIUS_PACKET *packet, RADIUS_PACKET *original, const char *secre
 				 *  User-Password
 				 */
 			case FLAG_ENCRYPT_USER_PASSWORD:
-				rad_pwdecode((char *)pair->strvalue,
-					     pair->length, secret,
-					     (char *)packet->vector);
-				pair->lvalue = 1; /* see main/auth.c */
-
-				/*
-				 *  FIXME: Allow NUL's in the password,
-				 *  too??
-				 */
-				pair->length = strlen(pair->strvalue);
+				if (original) {
+					rad_pwdecode((char *)pair->strvalue,
+						     pair->length, secret,
+						     (char *)original->vector);
+				} else {
+					rad_pwdecode((char *)pair->strvalue,
+						     pair->length, secret,
+						     (char *)packet->vector);
+				}
+				if (pair->attribute == PW_USER_PASSWORD) {
+					pair->length = strlen(pair->strvalue);
+				}
 				break;
 
 				/*
@@ -1375,11 +1424,13 @@ int rad_decode(RADIUS_PACKET *packet, RADIUS_PACKET *original, const char *secre
 
 	/*
 	 *	Merge information from the outside world into our
-	 *	random vector pool.
+	 *	random pool
 	 */
 	for (length = 0; length < AUTH_VECTOR_LEN; length++) {
-		random_vector_pool[length] ^= packet->vector[length];
+		lrad_rand_pool.randmem[length] += packet->vector[length];
 	}
+	lrad_rand_pool.randmem[lrad_rand_pool.randmem[0] & 0xff] += packet->id;
+	lrad_rand_pool.randmem[lrad_rand_pool.randmem[1] & 0xff] += packet->data_len;
 
 	return 0;
 }
@@ -1397,7 +1448,8 @@ int rad_decode(RADIUS_PACKET *packet, RADIUS_PACKET *original, const char *secre
  *	password - a multiple of 16 bytes.
  */
 #define AUTH_PASS_LEN (16)
-int rad_pwencode(char *passwd, int *pwlen, const char *secret, const char *vector)
+int rad_pwencode(char *passwd, int *pwlen, const char *secret,
+		 const char *vector)
 {
 	uint8_t	buffer[AUTH_VECTOR_LEN + MAX_STRING_LEN + 1];
 	char	digest[AUTH_VECTOR_LEN];
@@ -1405,9 +1457,9 @@ int rad_pwencode(char *passwd, int *pwlen, const char *secret, const char *vecto
 	int	len;
 
 	/*
-	 *	Padd password to multiple of AUTH_PASS_LEN bytes.
+	 *	Pad password to multiple of AUTH_PASS_LEN bytes.
 	 */
-	len = strlen(passwd);
+	len = *pwlen;
 	if (len > 128) len = 128;
 	*pwlen = len;
 	if (len % AUTH_PASS_LEN != 0) {
@@ -1450,7 +1502,8 @@ int rad_pwencode(char *passwd, int *pwlen, const char *secret, const char *vecto
 /*
  *	Decode password.
  */
-int rad_pwdecode(char *passwd, int pwlen, const char *secret, const char *vector)
+int rad_pwdecode(char *passwd, int pwlen, const char *secret,
+		 const char *vector)
 {
 	uint8_t	buffer[AUTH_VECTOR_LEN + MAX_STRING_LEN + 1];
 	char	digest[AUTH_VECTOR_LEN];
@@ -1497,6 +1550,7 @@ int rad_pwdecode(char *passwd, int pwlen, const char *secret, const char *vector
 	return pwlen;
 }
 
+static unsigned int salt_offset = 0;
 
 /*
  *	Encode Tunnel-Password attributes when sending them out on the wire.
@@ -1507,9 +1561,6 @@ int rad_pwdecode(char *passwd, int pwlen, const char *secret, const char *vector
  *      This is per RFC-2868 which adds a two char SALT to the initial intermediate
  *      value MD5 hash.
  */
-
-static int saltoffset = 0; /* used to make all salts in a packet unique */
-
 int rad_tunnel_pwencode(char *passwd, int *pwlen, const char *secret, const char *vector)
 {
 	uint8_t	buffer[AUTH_VECTOR_LEN + MAX_STRING_LEN + 3];
@@ -1537,10 +1588,21 @@ int rad_tunnel_pwencode(char *passwd, int *pwlen, const char *secret, const char
 	len += 1;
 
 
-	/* generate salt */
-	saltoffset++;
-	salt[0] = (vector[0] ^ vector[1] ^ 0x3A ^ (char) ( (saltoffset>>8) & 0x000000ff ) ) | 0x80;
-	salt[1] = (vector[2] ^ vector[3] ^ vector[4] ^ (char) (saltoffset & 0x000000ff) );
+	/*
+	 *	Generate salt.  The RFC's say:
+	 *
+	 *	high bit of salt[0] must be set.
+	 *	each salt in a packet should be unique.
+	 *	they should be random
+	 *
+	 *	So, we set the high bit,
+	 *	add in a counter,
+	 *	and then add in some CSPRNG data.
+	 *	should be OK..
+	 */
+	salt[0] = (0x80 | ( ((salt_offset++) & 0x0f) << 3) |
+		   (lrad_rand() & 0x07));
+	salt[1] = lrad_rand();
 	
 	/*
 	 *	Padd password to multiple of AUTH_PASS_LEN bytes.
@@ -1645,11 +1707,12 @@ int rad_tunnel_pwdecode(char *passwd, int * pwlen, const char *secret, const cha
  *	we use vp->length, and Ascend gear likes
  *	to send an extra '\0' in the string!
  */
-int rad_chap_encode(RADIUS_PACKET *packet, char *output, int id, VALUE_PAIR *password)
+int rad_chap_encode(RADIUS_PACKET *packet, char *output, int id,
+		    VALUE_PAIR *password)
 {
 	int		i;
 	char		*ptr;
-	char		string[MAX_STRING_LEN];
+	char		string[MAX_STRING_LEN * 2 + 1];
 	VALUE_PAIR	*challenge;
 
 	/*
@@ -1699,88 +1762,70 @@ int rad_chap_encode(RADIUS_PACKET *packet, char *output, int id, VALUE_PAIR *pas
  */
 static void random_vector(uint8_t *vector)
 {
-	int		i;
-	static int	did_srand = 0;
-	static int	counter = 0;
-#ifdef __linux__
-	static int	urandom_fd = -1;
+	int i;
 
-	/*
-	 *	Use /dev/urandom if available.
-	 */
-	if (urandom_fd > -2) {
-		/*
-		 *	Open urandom fd if not yet opened.
-		 */
-		if (urandom_fd < 0)
-			urandom_fd = open("/dev/urandom", O_RDONLY);
-		if (urandom_fd < 0) {
-			/*
-			 *	It's not there, don't try
-			 *	it again.
-			 */
-			DEBUG("Cannot open /dev/urandom, using rand()\n");
-			urandom_fd = -2;
-		} else {
-
-			fcntl(urandom_fd, F_SETFD, 1);
-
-			/*
-			 *	Read 16 bytes.
-			 */
-			if (read(urandom_fd, (char *) vector, AUTH_VECTOR_LEN)
-			    == AUTH_VECTOR_LEN)
-				return;
-			/*
-			 *	We didn't get 16 bytes - fall
-			 *	back on rand) and don't try again.
-			 */
-		DEBUG("Read short packet from /dev/urandom, using rand()\n");
-			urandom_fd = -2;
-		}
-	}
-#endif
-
-	if (!did_srand) {
-		srand(time(NULL) + getpid());
+	if (!lrad_pool_initialized) {
+		memset(&lrad_rand_pool, 0, sizeof(lrad_rand_pool));
 
 		/*
-		 *	Now that we have a bad random seed, let's
-		 *	make it a little better by MD5'ing it.
+		 *	Initialize the state to something, using
+		 *	numbers which aren't random, but which also
+		 *	aren't static.
 		 */
-		for (i = 0; i < (int)sizeof(random_vector_pool); i++) {
-			random_vector_pool[i] += rand() & 0xff;
-		}
+		lrad_rand_pool.randrsl[0] = (uint32_t) &lrad_pool_initialized;
+		lrad_rand_pool.randrsl[1] = (uint32_t) &i;
+		lrad_rand_pool.randrsl[2] = (uint32_t) vector;
 
-		librad_md5_calc((u_char *) random_vector_pool,
-				(u_char *) random_vector_pool,
-				sizeof(random_vector_pool));
-
-		did_srand = 1;
+		lrad_randinit(&lrad_rand_pool, 1);
 	}
 
-	/*
-	 *	Modify our random pool, based on the counter,
-	 *	and put the resulting information through MD5,
-	 *	so it's all mashed together.
-	 */
-	counter++;
-	random_vector_pool[AUTH_VECTOR_LEN] += (counter & 0xff);
-	librad_md5_calc((u_char *) random_vector_pool,
-			(u_char *) random_vector_pool,
-			sizeof(random_vector_pool));
+	lrad_isaac(&lrad_rand_pool);
 
 	/*
-	 *	And do another MD5 hash of the result, to give
-	 *	the user a random vector.  This ensures that the
-	 *	user has a random vector, without giving them
-	 *	an exact image of what's in the random pool.
+	 *	Copy the random data over.
 	 */
-	librad_md5_calc((u_char *) vector,
-			(u_char *) random_vector_pool,
-			sizeof(random_vector_pool));
+	for (i = 0; i < AUTH_VECTOR_LEN; i++) {
+		*(vector++) = lrad_rand_pool.randrsl[i] & 0xff;
+	}
 }
 
+/*
+ *	Return a 32-bit random number.
+ */
+uint32_t lrad_rand(void)
+{
+	uint32_t answer;
+	static int rand_index = 0;
+
+	/*
+	 *	Ensure that the pool is initialized.
+	 */
+	if (!lrad_pool_initialized) {
+		uint8_t vector[AUTH_VECTOR_LEN];
+
+		random_vector(vector);
+	}
+
+	/*
+	 *	Grab an entry from the pool.
+	 */
+	answer = lrad_rand_pool.randrsl[rand_index];
+
+	/*
+	 *	Go to the next entry (wrapping around to zero).
+	 */
+	rand_index++;
+	rand_index &= 0xff;
+
+	/*
+	 *	Every 256 numbers, churn the pool again.
+	 */
+	if (rand_index == 0) {
+		lrad_isaac(&lrad_rand_pool);
+	}
+
+	return answer;
+}
 
 /*
  *	Allocate a new RADIUS_PACKET
