@@ -1,7 +1,7 @@
 /*
  * radiusd.c	Main loop of the radius server.
  *
- * Version:	$Id: radiusd.c,v 1.272.2.1 2002/11/29 16:51:47 aland Exp $
+ * Version:	$Id: radiusd.c,v 1.288.2.1 2003/07/16 14:42:50 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -27,7 +27,7 @@
 
 /* don't look here for the version, run radiusd -v or look in version.c */
 static const char rcsid[] =
-"$Id: radiusd.c,v 1.272.2.1 2002/11/29 16:51:47 aland Exp $";
+"$Id: radiusd.c,v 1.288.2.1 2003/07/16 14:42:50 aland Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -79,7 +79,7 @@ static const char rcsid[] =
 #include "modules.h"
 #include "request_list.h"
 
-#if WITH_SNMP
+#ifdef WITH_SNMP
 #	include "radius_snmp.h"
 #endif
 
@@ -100,6 +100,7 @@ int auth_port = 0;
 int acct_port;
 int proxy_port;
 int need_reload = FALSE;
+int sig_hup_block = FALSE;
 const char *radiusd_version = "FreeRADIUS Version " RADIUSD_VERSION ", for host " HOSTINFO ", built on " __DATE__ " at " __TIME__;
 
 static int got_child = FALSE;
@@ -126,6 +127,9 @@ static void usage(void);
 
 static void sig_fatal (int);
 static void sig_hup (int);
+#ifdef HAVE_PTHREAD_H
+static void sig_cleanup(int);
+#endif
 
 static void rfc_clean(RADIUS_PACKET *packet);
 static void rad_reject(REQUEST *request);
@@ -133,10 +137,12 @@ static struct timeval *rad_clean_list(time_t curtime);
 static REQUEST *rad_check_list(REQUEST *);
 static REQUEST *proxy_check_list(REQUEST *request);
 static int refresh_request(REQUEST *request, void *data);
-#ifndef HAVE_PTHREAD_H
-static int rad_spawn_child(REQUEST *, RAD_REQUEST_FUNP);
-#else
+#ifdef HAVE_PTHREAD_H
 extern int rad_spawn_child(REQUEST *, RAD_REQUEST_FUNP);
+#else
+#ifdef ALLOW_CHILD_FORKS
+static int rad_spawn_child(REQUEST *, RAD_REQUEST_FUNP);
+#endif
 #endif
 static int rad_status_server(REQUEST *request);
 
@@ -274,6 +280,9 @@ int main(int argc, char *argv[])
 	int radius_port = 0;
 	struct servent *svp;
 	struct timeval *tv = NULL;
+#ifdef HAVE_SIGACTION
+	struct sigaction act;
+#endif
 
 	syslog_facility = LOG_DAEMON;
 
@@ -294,6 +303,11 @@ int main(int argc, char *argv[])
 	 *	Ensure that the configuration is initialized.
 	 */
 	memset(&mainconfig, 0, sizeof(mainconfig));
+#ifdef HAVE_SIGACTION
+	memset(&act, 0, sizeof(act));
+	act.sa_flags = 0 ;
+	sigemptyset( &act.sa_mask ) ;
+#endif
 
 	/*  Process the options.  */
 	while ((argval = getopt(argc, argv, "Aa:bcd:fg:hi:l:p:sSvxXyz")) != EOF) {
@@ -415,7 +429,12 @@ int main(int argc, char *argv[])
 	 */
 	if ((mainconfig.allow_core_dumps == FALSE) && (debug_flag == 0)) {
 #ifdef SIGSEGV
+#ifdef HAVE_SIGACTION
+		act.sa_handler = sig_fatal;
+		sigaction(SIGSEGV, &act, NULL);
+#else
 		signal(SIGSEGV, sig_fatal);
+#endif
 #endif
 	}
 
@@ -539,7 +558,6 @@ int main(int argc, char *argv[])
 		/*
 		 *  Open Proxy Socket.
 		 */
-		check_proxies(mainconfig.max_proxies);
 		proxyfd = socket (AF_INET, SOCK_DGRAM, 0);
 		if (proxyfd < 0) {
 			perror ("proxy socket");
@@ -584,8 +602,8 @@ int main(int argc, char *argv[])
 	 */
 	pair_builtincompare_init();
 
-#if WITH_SNMP
-	radius_snmp_init();
+#ifdef WITH_SNMP
+	if (mainconfig.do_snmp) radius_snmp_init();
 #endif
 
 	/*
@@ -668,6 +686,11 @@ int main(int argc, char *argv[])
 	}
 
 	rad_exec_init();
+#else
+	/*
+	 *	Without threads, we ALWAYS run in single-server mode.
+	 */
+	spawn_flag = FALSE;
 #endif
 
 	/*
@@ -696,24 +719,54 @@ int main(int argc, char *argv[])
 	 *	handlers.  Before this, if we get any signal, we don't know
 	 *	what to do, so we might as well do the default, and die.
 	 */
-	signal(SIGHUP, sig_hup);
 	signal(SIGPIPE, SIG_IGN);	
+#ifdef HAVE_SIGACTION
+	act.sa_handler = sig_hup;
+	sigaction(SIGHUP, &act, NULL);
+	act.sa_handler = sig_fatal;
+	sigaction(SIGTERM, &act, NULL);
+#else
+	signal(SIGHUP, sig_hup);
 	signal(SIGTERM, sig_fatal);
-
+#endif
 	/*
 	 *	If we're debugging, then a CTRL-C will cause the
 	 *	server to die immediately.  Use SIGTERM to shut down
 	 *	the server cleanly in that case.
 	 */
 	if (debug_flag == 0) {
+#ifdef HAVE_SIGACTION
+	        act.sa_handler = sig_fatal;
+		sigaction(SIGINT, &act, NULL);
+		sigaction(SIGQUIT, &act, NULL);
+#else
 		signal(SIGINT, sig_fatal);
 		signal(SIGQUIT, sig_fatal);
+#endif
 	}
 
-	/* this is right for threads, too, right?  
-	 * (Threads shouldn't get signals (from us) -- just be pthread_cancel()led.)
+#ifdef HAVE_PTHREAD_H
+	/*
+	 *	If we have pthreads, then the child threads block
+	 *	SIGCHLD, and the main server thread catches it.
+	 *
+	 *	That way, the SIGCHLD handler can grab the exit status,
+	 *	and save it for the child thread.
+	 *
+	 *	If we don't have pthreads, then each child process
+	 *	will do a waitpid(), and we ignore SIGCHLD.
+	 *
+	 *	Once we have multiple child processes to handle
+	 *	requests, and shared memory, then we've got to
+	 *	re-enable SIGCHLD catching.
 	 */
+#ifdef HAVE_SIGACTION
+	act.sa_handler = sig_cleanup;
+	sigaction(SIGCHLD, &act, NULL);
+#else
 	signal(SIGCHLD, sig_cleanup);
+#endif
+#endif
 
 	radlog(L_INFO, "Ready to process requests.");
 	start_time = time(NULL);
@@ -782,6 +835,21 @@ int main(int argc, char *argv[])
 		}
 
 		if (need_reload) {
+		        int max_wait = 0;
+		        for(;;) {
+			        /*
+				 * Block until there are '0' threads
+				 * with a REQUEST handle.
+				 */
+			        sig_hup_block = TRUE;
+			        if( (total_active_threads() == 0) ||
+				     (max_wait >= 5) ) {
+				  sig_hup_block = FALSE;
+				  break;
+				}
+				sleep(1);
+				max_wait++;
+			}
 			if (read_mainconfig(TRUE) < 0) {
 				exit(1);
 			}
@@ -812,13 +880,26 @@ int main(int argc, char *argv[])
 			if (proxyfd > max_fd) max_fd = proxyfd;
 		}
 #ifdef WITH_SNMP
-		if (rad_snmp.smux_fd >= 0) {
+		if (mainconfig.do_snmp &&
+		    (rad_snmp.smux_fd >= 0)) {
 			FD_SET(rad_snmp.smux_fd, &readfds);
 			if (rad_snmp.smux_fd > max_fd) max_fd = rad_snmp.smux_fd;
 		}
 #endif
 
 		status = select(max_fd + 1, &readfds, NULL, NULL, tv);
+#ifndef HAVE_PTHREAD_H
+		/*
+		 *	If there are no child threads, then there may
+		 *	be child processes.  In that case, wait for
+		 *	their exit status, and throw that exit status
+		 *	away.  This helps get rid of zxombie children.
+		 */
+		while (waitpid(-1, &argval, WNOHANG) > 0) {
+			/* do nothing */
+		}
+#endif
+
 		if (status == -1) {
 			/*
 			 *	On interrupts, we clean up the request
@@ -849,16 +930,21 @@ int main(int argc, char *argv[])
 			/*
 			 *  Receive the packet.
 			 */
+			if (sig_hup_block != FALSE) {
+			  continue;
+			}
 			packet = rad_recv(fd);
 			if (packet == NULL) {
 				radlog(L_ERR, "%s", librad_errstr);
 				continue;
 			}
-#if WITH_SNMP
-			if (fd == acctfd)
-				rad_snmp.acct.total_requests++;
-			if (fd == authfd)
-				rad_snmp.auth.total_requests++;
+#ifdef WITH_SNMP
+			if (mainconfig.do_snmp) {
+				if (fd == acctfd)
+					rad_snmp.acct.total_requests++;
+				if (fd == authfd)
+					rad_snmp.auth.total_requests++;
+			}
 #endif
 
 			/*
@@ -881,7 +967,7 @@ int main(int argc, char *argv[])
 			} else {    /* It came in on the proxy port */
 				REALM *rl;
 				if ((rl = realm_findbyaddr(packet->src_ipaddr,packet->src_port)) == NULL) {
-					radlog(L_ERR, "Ignoring request from unknown proxy %s:%d",
+					radlog(L_ERR, "Ignoring request from unknown home server %s:%d",
 					ip_ntoa((char *)buffer, packet->src_ipaddr),
 					packet->src_port);
 					rad_free(&packet);
@@ -926,28 +1012,30 @@ int main(int argc, char *argv[])
 			rad_process(request, spawn_flag);
 		} /* loop over authfd, acctfd, proxyfd */
 
-#if WITH_SNMP
-		/*
-		 *  After handling all authentication/accounting
-		 *  requests, THEN process any pending SMUX/SNMP
-		 *  queries.
-		 *
-		 *  Note that the handling is done in the main server,
-		 *  which probably isn't a Good Thing.  It really
-		 *  should be wrapped, and handled in a thread pool.
-		 */
-		if ((rad_snmp.smux_fd >= 0) &&
-		    FD_ISSET(rad_snmp.smux_fd, &readfds) &&
-		    (rad_snmp.smux_event == SMUX_READ)) {
-			smux_read();
-		}
-		
-		/*
-		 *  If we've got to re-connect, then do so now,
-		 *  before calling select again.
-		 */
-		if (rad_snmp.smux_event == SMUX_CONNECT) {
-			smux_connect();
+#ifdef WITH_SNMP
+		if (mainconfig.do_snmp) {
+			/*
+			 *  After handling all authentication/accounting
+			 *  requests, THEN process any pending SMUX/SNMP
+			 *  queries.
+			 *
+			 *  Note that the handling is done in the main server,
+			 *  which probably isn't a Good Thing.  It really
+			 *  should be wrapped, and handled in a thread pool.
+			 */
+			if ((rad_snmp.smux_fd >= 0) &&
+			    FD_ISSET(rad_snmp.smux_fd, &readfds) &&
+			    (rad_snmp.smux_event == SMUX_READ)) {
+				smux_read();
+			}
+			
+			/*
+			 *  If we've got to re-connect, then do so now,
+			 *  before calling select again.
+			 */
+			if (rad_snmp.smux_event == SMUX_CONNECT) {
+				smux_connect();
+			}
 		}
 #endif
 
@@ -1119,6 +1207,14 @@ int rad_process(REQUEST *request, int dospawn)
 		request->reply->data_len = 0;
 	}
 
+	/*
+	 *	If we don't have threads, then the child CANNOT save
+	 *	it's state in the memory used by the main server core.
+	 *
+	 *	That is, until someone goes and implements shared
+	 *	memory across processes...
+	 */
+#ifdef HAVE_PTHREAD_H
 	/*
 	 *  If we're spawning a child thread, let it do all of
 	 *  the work of handling a request, and exit.
@@ -1135,6 +1231,7 @@ int rad_process(REQUEST *request, int dospawn)
 		}
 		return 0;
 	}
+#endif
 
 	rad_respond(request, fun);
 	return 0;
@@ -1344,17 +1441,44 @@ int rad_respond(REQUEST *request, RAD_REQUEST_FUNP fun)
 	 *  attributes from the list of VP's.
 	 */
 	if (request->proxy) {
-		int replicating;
-		replicating = proxy_receive(request);
-              switch (replicating) {
-                      case -1:
-                              /* on error just continue with next request */
-                              goto next_request;
-                      case 1:
-                              /* if this was a replicated request, mark it as
-                               * finished first, because it was postponed
-                               */
-                              goto finished_request;
+            int rcode;
+            rcode = proxy_receive(request);
+            switch (rcode) {
+                default:  /* Don't Do Anything */
+                    break;
+                case RLM_MODULE_FAIL:
+                    /* on error just continue with next request */
+                    goto next_request;
+                case RLM_MODULE_HANDLED:
+                    /* if this was a replicated request, mark it as
+                     * finished first, because it was postponed
+                     */
+                    goto finished_request;
+            }
+
+	} else {
+		/*
+		 *	This is the initial incoming request which
+		 *	we're processing.
+		 *
+		 *	Some requests do NOT get cached, as they
+		 *	CANNOT possibly have duplicates.  Set the
+		 *	magic option here.
+		 *
+		 *	Status-Server messages are easy to generate,
+		 *	so we toss them as soon as we see a reply.
+		 *
+		 *	Accounting-Request packets WITHOUT an
+		 *	Acct-Delay-Time attribute are NEVER
+		 *	duplicated, as RFC 2866 Section 4.1 says that
+		 *	the Acct-Delay-Time MUST be updated when the
+		 *	packet is re-sent, which means the packet
+		 *	changes, so it MUST have a new identifier and
+		 *	Request Authenticator.  */
+		if ((request->packet->code == PW_STATUS_SERVER) ||
+		    ((request->packet->code == PW_ACCOUNTING_REQUEST) &&
+		     (pairfind(request->packet->vps, PW_ACCT_DELAY_TIME) == NULL))) {
+			request->options |= RAD_REQUEST_OPTION_DONT_CACHE;
 		}
 	}
 	
@@ -1381,12 +1505,14 @@ int rad_respond(REQUEST *request, RAD_REQUEST_FUNP fun)
 	if(strcmp(mainconfig.do_lower_user, "before") == 0)
 		rad_lowerpair(request, request->username);
 	if(strcmp(mainconfig.do_lower_pass, "before") == 0)
-		rad_lowerpair(request, rad_getpass(request));
+		rad_lowerpair(request,
+			      pairfind(request->packet->vps, PW_PASSWORD));
 
 	if(strcmp(mainconfig.do_nospace_user, "before") == 0)
 		rad_rmspace_pair(request, request->username);
 	if(strcmp(mainconfig.do_nospace_pass, "before") == 0)
-		rad_rmspace_pair(request, rad_getpass(request));
+		rad_rmspace_pair(request,
+				 pairfind(request->packet->vps, PW_PASSWORD));
 
 	(*fun)(request);
 
@@ -1401,7 +1527,8 @@ int rad_respond(REQUEST *request, RAD_REQUEST_FUNP fun)
 		  reprocess = 1;
 	  }
 	  if (strcmp(mainconfig.do_lower_pass, "after") == 0) {
-		rad_lowerpair(request, rad_getpass(request));
+		rad_lowerpair(request,
+			      pairfind(request->packet->vps, PW_PASSWORD));
 		reprocess = 1;
 	  }
 	  if (strcmp(mainconfig.do_nospace_user, "after") == 0) {
@@ -1409,12 +1536,19 @@ int rad_respond(REQUEST *request, RAD_REQUEST_FUNP fun)
 		  reprocess = 1;
 	  }
 	  if (strcmp(mainconfig.do_nospace_pass, "after") == 0) {
-		  rad_rmspace_pair(request, rad_getpass(request));
+		  rad_rmspace_pair(request,
+				   pairfind(request->packet->vps, PW_PASSWORD));
+
 		  reprocess = 1;
 	  }
 	  
+	  /*
+	   *	If we're re-processing the request, re-set it.
+	   */
 	  if (reprocess) {
 		  pairfree(&request->config_items);
+		  pairfree(&request->reply->vps);
+		  request->reply->code = 0;
 		  (*fun)(request);
 	  }
 	}
@@ -1435,23 +1569,41 @@ int rad_respond(REQUEST *request, RAD_REQUEST_FUNP fun)
 			 */
 			rcode = proxy_send(request);
 
+			switch (rcode) {
+			default:
+				break;
+				
 			/*
 			 *  There was an error trying to proxy the request.
 			 *  Drop it on the floor.
 			 */
-			if (rcode == RLM_MODULE_FAIL) {
+			case RLM_MODULE_FAIL:
 				DEBUG2("Error trying to proxy request %d: Rejecting it", request->number);
 				rad_reject(request);
 				goto finished_request;
-			}
-			
+				break;
+
 			/*
-			 *  If thie proxy code has handled the request,
+			 *  The pre-proxy module has decided to reject
+			 *  the request.  Do so.
+			 */
+			case RLM_MODULE_REJECT:
+				DEBUG2("Request %d rejected in proxy_send.", request->number);
+				rad_reject(request);
+				goto finished_request;
+				break;
+				
+			/*
+			 *  If the proxy code has handled the request,
 			 *  then postpone more processing, until we get
 			 *  the reply packet from the home server.
 			 */
-			if (rcode == RLM_MODULE_HANDLED) {
+			case RLM_MODULE_HANDLED:
+				/*
+				 *  rad_send??
+				 */
 				goto postpone_request;
+				break;
 			}
 
 			/*
@@ -1564,10 +1716,12 @@ finished_request:
 
 	DEBUG2("Finished request %d", request->number);
 	finished = TRUE;
-	
+
 	/*
 	 *  Go to the next request, without marking
 	 *  the current one as finished.
+	 *
+	 *  Hmm... this may not be the brightest thing to do.
 	 */
 next_request:
 	DEBUG2("Going to the next request");
@@ -1901,10 +2055,12 @@ static REQUEST *rad_check_list(REQUEST *request)
 			 *	Therefore, we MUST be waiting for a reply
 			 *	from the proxy.
 			 *
-			 *	If not, then we have no clue what to
-			 *	do, so we drop the new request, and
-			 *	hope that the NAS doesn't bug us about
-			 *	it.
+			 *	If not, then it's an Accounting-Request
+			 *	which we tried to "reject", which means
+			 *	that we silently drop the response.
+			 *      We want to give the same response for the
+			 *      duplicate request, so we silently drop it,
+			 *	too.
 			 */
 			if (!curreq->proxy) {
 				radlog(L_ERR, "Dropping packet from client "
@@ -1961,10 +2117,15 @@ static REQUEST *rad_check_list(REQUEST *request)
 			 *	In that case, go kick the home RADIUS
 			 *	server again.
 			 */
-			DEBUG2("Sending duplicate proxied request to client %s:%d - ID: %d",
-			       client_name(curreq->proxy->dst_ipaddr), request->packet->src_port,
-			       curreq->proxy->id);
-			
+			{
+				char buffer[32];
+
+				DEBUG2("Sending duplicate proxied request to home server %s:%d - ID: %d",
+				       ip_ntoa(buffer, curreq->proxy->dst_ipaddr),
+				       curreq->proxy->dst_port,
+									
+				       curreq->proxy->id);
+			}
 			curreq->proxy_next_try = request->timestamp + mainconfig.proxy_retry_delay;
 			rad_send(curreq->proxy, curreq->packet, curreq->proxysecret);
 			request_free(&request);
@@ -2028,8 +2189,13 @@ static REQUEST *rad_check_list(REQUEST *request)
  *  'threads.c' replaces this one.
  *
  *  This code is NOT well tested, and should NOT be used!
+ *
+ *  Note that ALLOW_CHILD_FORKS is never defined.  This code
+ *  is here for historical purposes, so that if (or when) someone
+ *  implements shared memory between processes, that we have a template
+ *  of code to work from.
  */
-#ifndef HAVE_PTHREAD_H
+#ifdef ALLOW_CHILD_FORKS
 /*
  *  Spawns a child process or thread to perform
  *  authentication/accounting and respond to RADIUS clients.
@@ -2041,7 +2207,17 @@ static int rad_spawn_child(REQUEST *request, RAD_REQUEST_FUNP fun)
 
 	/* spawning and registering a child is a critical section, so
 	 * we refuse to handle SIGCHLDs normally until we're finished. */
+#ifdef HAVE_SIGACTION
+	struct sigaction act;
+
+	memset(&act, 0, sizeof(act));
+	act.sa_flags = 0 ;
+	sigemptyset( &act.sa_mask ) ;
+	act.sa_handler = queue_sig_cleanup;
+	sigaction(SIGCHLD, &act, NULL);
+#else
 	signal(SIGCHLD, queue_sig_cleanup);
+#endif
 
 	/*
 	 *  fork our child
@@ -2071,7 +2247,12 @@ static int rad_spawn_child(REQUEST *request, RAD_REQUEST_FUNP fun)
 	request->child_pid = child_pid;
 
 exit_child_critsec:
+#ifdef HAVE_SIGACTION
+	act.sa_handler = sig_cleanup;
+	sigaction(SIGCHLD, &act, NULL);
+#else
 	signal(SIGCHLD, sig_cleanup);
+#endif
 	if (needs_child_cleanup > 0) {
 		sig_cleanup(0);
 	}
@@ -2095,15 +2276,15 @@ void queue_sig_cleanup(int sig) {
 	needs_child_cleanup++;
 	return;
 }
-#endif /* HAVE_PTHREAD_H */
+#endif /* ALLOW_CHILD_FORKS */
 
 
-/*ARGSUSED*/
-void sig_cleanup(int sig)
+#ifdef HAVE_PTHREAD_H
+static void sig_cleanup(int sig)
 {
 	int status;
 	child_pid_t pid;
-#ifndef HAVE_PTHREAD_H
+#ifdef ALLOW_CHILD_FORKS
 	REQUEST *curreq;
 #endif
 
@@ -2147,6 +2328,7 @@ void sig_cleanup(int sig)
 #ifdef HAVE_PTHREAD_H
 		rad_savepid(pid, status);
 #else
+#ifdef ALLOW_CHILD_FORKS
 		/*
 		 *  Loop over ALL of the active requests, looking
 		 *  for the one which caused the signal.
@@ -2154,9 +2336,11 @@ void sig_cleanup(int sig)
 		if (rl_walk(sig_cleanup_walker, (void*)pid) != 0) {
 			radlog(L_ERR, "Failed to cleanup child %d", pid);
 		}
+#endif
 #endif /* !defined HAVE_PTHREAD_H */
 	}
 }
+#endif /* HAVE_PTHREAD_H */
 
 /*
  *  Display the syntax for starting this program.
@@ -2221,8 +2405,10 @@ static void sig_hup(int sig)
 		need_reload = TRUE;
 	}
 #ifdef WITH_SNMP
-	rad_snmp.smux_failures = 0;
-	rad_snmp.smux_event = SMUX_CONNECT;
+	if (mainconfig.do_snmp) {
+		rad_snmp.smux_failures = 0;
+		rad_snmp.smux_event = SMUX_CONNECT;
+	}
 #endif
 }
 
@@ -2231,7 +2417,9 @@ static void sig_hup(int sig)
  */
 static REQUEST *proxy_check_list(REQUEST *request)
 {
+	REALM *cl;
 	REQUEST *oldreq;
+	char buffer[32];
 	
 	/*
 	 *	Find the original request in the request list
@@ -2244,8 +2432,8 @@ static REQUEST *proxy_check_list(REQUEST *request)
 	 *	request, as there's no way for us to send it to a NAS.
 	 */
 	if (!oldreq) {
-		radlog(L_PROXY, "No matching request was found for proxy reply from server %s:%d - ID %d",
-		       client_name(request->packet->src_ipaddr),
+		radlog(L_PROXY, "No outstanding request was found for proxy reply from home server %s:%d - ID %d",
+		       ip_ntoa(buffer, request->packet->src_ipaddr),
 		       request->packet->src_port,
 		       request->packet->id);
 		request_free(&request);
@@ -2258,7 +2446,7 @@ static REQUEST *proxy_check_list(REQUEST *request)
 	 */
 	if (oldreq->child_pid != NO_SUCH_CHILD_PID) {
 		radlog(L_ERR, "Discarding duplicate reply from home server %s:%d - ID: %d due to live request %d",
-		       client_name(request->packet->src_ipaddr),
+		       ip_ntoa(buffer, request->packet->src_ipaddr),
 		       request->packet->src_port,
 		       request->packet->id,
 		       oldreq->number);
@@ -2276,7 +2464,7 @@ static REQUEST *proxy_check_list(REQUEST *request)
 	if ((oldreq->reply->code != 0) ||
 	    (oldreq->finished)) {
 		radlog(L_ERR, "Reply from home server %s:%d arrived too late for request %d. Try increasing 'retry_delay' or 'max_request_time'",
-		       client_name(request->packet->src_ipaddr),
+		       ip_ntoa(buffer, request->packet->src_ipaddr),
 		       request->packet->src_port,
 		       oldreq->number);
 		request_free(&request);
@@ -2320,6 +2508,32 @@ static REQUEST *proxy_check_list(REQUEST *request)
 	oldreq->proxy_reply = request->packet;
 	request->packet = NULL;
 	request_free(&request);
+
+	/*
+	 *	Now that we've verified the packet IS actually
+	 *	from that realm, and not forged, we can go mark the
+	 *	realms for this home server as active.
+	 *
+	 *	If we had done this check in the 'find realm by IP address'
+	 *	function, then an attacker could force us to use a home
+	 *	server which was inactive, by forging reply packets
+	 *	which didn't match any request.  We would think that
+	 *	the reply meant the home server was active, would
+	 *	re-activate the realms, and THEN bounce the packet
+	 *	as garbage.
+	 */
+	for (cl = mainconfig.realms; cl != NULL; cl = cl->next) {
+		if (oldreq->proxy_reply->src_ipaddr == cl->ipaddr) {
+			if (oldreq->proxy_reply->src_port == cl->auth_port) {
+				cl->active = TRUE;
+				cl->last_reply = oldreq->timestamp;
+			} else if (oldreq->proxy_reply->src_port == cl->acct_port) {
+				cl->acct_active = TRUE;
+				cl->last_reply = oldreq->timestamp;
+			}
+		}
+	}
+
 	return oldreq;
 }
 
@@ -2364,15 +2578,13 @@ static int refresh_request(REQUEST *request, void *data)
 	 *  been cleaned up, AND it's time to clean up the request,
 	 *  OR, it's an accounting request.  THEN, go delete it.
 	 *
-	 *  If this is an accounting or Status-Server request, we
-	 *  delete it immediately, as there CANNOT be duplicate
-	 *  accounting packets.  If there are, then something else is
-	 *  seriously wrong...
+	 *  If this is a request which had the "don't cache" option
+	 *  set, then delete it immediately, as it CANNOT have a
+	 *  duplicate.
 	 */
 	if (request->finished &&
 	    ((request->timestamp + mainconfig.cleanup_delay <= info->now) ||
-	     (request->packet->code == PW_STATUS_SERVER) ||
-	     (request->packet->code == PW_ACCOUNTING_REQUEST))) {
+	     ((request->options & RAD_REQUEST_OPTION_DONT_CACHE) != 0))) {
 		rad_assert(request->child_pid == NO_SUCH_CHILD_PID);
 		/*
 		 *  Request completed, delete it, and unlink it
@@ -2438,9 +2650,11 @@ static int refresh_request(REQUEST *request, void *data)
 				       request->number);
 				pthread_cancel(child_pid);
 #else
+#ifdef ALLOW_CHILD_FORKS
 				radlog(L_ERR, "Killing unresponsive child %lu for request %d",
 				       child_pid, request->number);
 				kill(child_pid, SIGTERM);
+#endif
 #endif
 			} /* else no proxy reply, quietly fail */
 		
@@ -2527,6 +2741,7 @@ static int refresh_request(REQUEST *request, void *data)
 	 *  the request as finished, and go to the next one.
 	 */
 	if (request->proxy_try_count == 0) {
+		rad_assert(request->child_pid == NO_SUCH_CHILD_PID);
 		rad_reject(request);
 		realm_disable(request->proxy->dst_ipaddr,request->proxy->dst_port);
 		request->finished = TRUE;
