@@ -1,7 +1,7 @@
 /*
  * session.c	session management
  *
- * Version:	$Id: session.c,v 1.15.2.2 2002/12/03 15:26:53 aland Exp $
+ * Version:	$Id: session.c,v 1.20 2003/06/20 17:50:10 phampson Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -43,7 +43,7 @@
 #include	"modules.h"
 
 /* End a session by faking a Stop packet to all accounting modules */
-int session_zap(int sockfd, uint32_t nasaddr, int port, const char *user,
+int session_zap(int sockfd, uint32_t nasaddr, unsigned int port, const char *user,
 		const char *sessionid, uint32_t cliaddr, char proto, time_t t)
 {
 	static unsigned char id = 0;
@@ -99,7 +99,7 @@ int session_zap(int sockfd, uint32_t nasaddr, int port, const char *user,
 	INTPAIR(PW_ACCT_DELAY_TIME, 0);
 	STRINGPAIR(PW_USER_NAME, user);
 	userpair = vp;
-	INTPAIR(PW_NAS_PORT_ID, port);
+	INTPAIR(PW_NAS_PORT, port);
 	STRINGPAIR(PW_ACCT_SESSION_ID, sessionid);
 	if(proto == 'P') {
 		INTPAIR(PW_SERVICE_TYPE, PW_FRAMED_USER);
@@ -169,7 +169,7 @@ int session_zap(int sockfd, uint32_t nasaddr, int port, const char *user,
 /*
  *	Check one terminal server to see if a user is logged in.
  */
-int rad_check_ts(uint32_t nasaddr, int portnum, const char *user,
+int rad_check_ts(uint32_t nasaddr, unsigned int portnum, const char *user,
 		 const char *session_id)
 {
 	pid_t	pid, child_pid;
@@ -187,6 +187,8 @@ int rad_check_ts(uint32_t nasaddr, int portnum, const char *user,
 		/*
 		 *  Unknown NAS, so trusting radutmp.
 		 */
+		DEBUG2("checkrad: Unknown NAS %s, not checking",
+		       ip_ntoa(address, nasaddr));
 		return 1;
 	}
 
@@ -195,6 +197,7 @@ int rad_check_ts(uint32_t nasaddr, int portnum, const char *user,
 	 */
 	if ((cl->nastype[0] == '\0') ||
 	    (strcmp(cl->nastype, "other") == 0)) {
+		DEBUG2("checkrad: No NAS type, or type \"other\" not checking");
 		return 1;
 	}
 
@@ -202,7 +205,7 @@ int rad_check_ts(uint32_t nasaddr, int portnum, const char *user,
 	 *	Fork.
 	 */
 	if ((pid = rad_fork(1)) < 0) { /* do wait for the fork'd result */
-		radlog(L_ERR, "Accounting: fork: %s", strerror(errno));
+		radlog(L_ERR, "Accounting: Failed in fork(): Cannot run checkrad\n");
 		return -1;
 	}
 
@@ -224,14 +227,13 @@ int rad_check_ts(uint32_t nasaddr, int portnum, const char *user,
 		}
 
 		/*
-		 *  It's taking too long.  Kill it.
+		 *	It's taking too long.  Stop waiting for it.
+		 *
+		 *	Don't bother to kill it, as we don't care what
+		 *	happens to it now.
 		 */
 		if (!found) {
-			kill(pid, SIGTERM);
-			sleep(1);
-			kill(pid, SIGKILL);
 			radlog(L_ERR, "Check-TS: timeout waiting for checkrad");
-			rad_waitpid(pid, &status, WNOHANG); /* to be safe */
 			return 2;
 		}
 
@@ -249,8 +251,17 @@ int rad_check_ts(uint32_t nasaddr, int portnum, const char *user,
 	for (n = 256; n >= 3; n--)
 		close(n);
 
+	/*
+	 *  We don't close fd's 0, 1, and 2.  If we're in debugging mode,
+	 *  then they should go to stdout (etc), along with the other
+	 *  server log messages.
+	 *
+	 *  If we're not in debugging mode, then the code in radiusd.c
+	 *  takes care of connecting fd's 0, 1, and 2 to /dev/null.
+	 */
+
 	ip_ntoa(address, nasaddr);
-	sprintf(port, "%d", portnum);
+	sprintf(port, "%u", portnum);
 
 #ifdef __EMX__
 	/* OS/2 can't directly execute scripts then we call the command
