@@ -1,7 +1,7 @@
 /*
  * version.c	Print version number and exit.
  *
- * Version:	$Id: version.c,v 1.11 2001/09/12 22:39:17 cmiller Exp $
+ * Version:	$Id: version.c,v 1.12 2003/06/22 07:47:41 phampson Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  * Copyright 2000  Chris Parker <cparker@starnetusa.com>
  */
 
-static const char rcsid[] = "$Id: version.c,v 1.11 2001/09/12 22:39:17 cmiller Exp $";
+static const char rcsid[] = "$Id: version.c,v 1.12 2003/06/22 07:47:41 phampson Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -45,9 +45,6 @@ void version(void)
 #if defined(OSFC2)
 	fprintf(stderr," OSFC2");
 #endif
-#if defined(WITH_DICT_NOCASE)
-	fprintf(stderr," WITH_DICT_NOCASE");
-#endif
 #if defined(WITH_SNMP)
 	fprintf(stderr," WITH_SNMP");
 #endif
