@@ -1,7 +1,7 @@
 /*
  * rlm_counter.c
  *
- * Version:  $Id: rlm_counter.c,v 1.27 2002/09/06 11:45:38 kkalev Exp $
+ * Version:  $Id: rlm_counter.c,v 1.32.2.1 2003/07/11 19:51:34 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -54,7 +54,9 @@
 #define gdbm_fdesc(foo) (-1)
 #endif
 
-static const char rcsid[] = "$Id: rlm_counter.c,v 1.27 2002/09/06 11:45:38 kkalev Exp $";
+#define UNIQUEID_MAX_LEN 32
+
+static const char rcsid[] = "$Id: rlm_counter.c,v 1.32.2.1 2003/07/11 19:51:34 aland Exp $";
 
 /*
  *	Define a structure for our module configuration.
@@ -75,6 +77,7 @@ typedef struct rlm_counter_t {
 	int service_val;
 	int key_attr;
 	int count_attr;
+	int check_attr;
 	time_t reset_time;	/* The time of the next reset. */
 	time_t last_reset;	/* The time of the last reset. */
 	int dict_attr;		/* attribute number for the counter. */
@@ -82,6 +85,11 @@ typedef struct rlm_counter_t {
 	pthread_mutex_t mutex;	/* A mutex to lock the gdbm file for only one reader/writer */
 } rlm_counter_t;
 
+typedef struct rad_counter {
+	unsigned int user_counter;
+	char uniqueid[UNIQUEID_MAX_LEN];
+} rad_counter;
+
 /*
  *	A mapping of configuration file names to internal variables.
  *
@@ -114,7 +122,7 @@ static int counter_cmp(void *instance, REQUEST *req, VALUE_PAIR *request, VALUE_
 	datum key_datum;
 	datum count_datum;
 	VALUE_PAIR *key_vp;
-	int counter;
+	rad_counter counter;
 
 	check_pairs = check_pairs; /* shut the compiler up */
 	reply_pairs = reply_pairs;
@@ -135,10 +143,10 @@ static int counter_cmp(void *instance, REQUEST *req, VALUE_PAIR *request, VALUE_
 	if (count_datum.dptr == NULL) {
 		return -1;
 	}
-	memcpy(&counter, count_datum.dptr, sizeof(int));
+	memcpy(&counter, count_datum.dptr, sizeof(rad_counter));
 	free(count_datum.dptr);
 
-	return counter - check->lvalue;
+	return counter.user_counter - check->lvalue;
 }
 
 static int add_defaults(rlm_counter_t *data)
@@ -303,6 +311,10 @@ static int counter_instantiate(CONF_SECTION *conf, void **instance)
 	 *	Set up a storage area for instance data
 	 */
 	data = rad_malloc(sizeof(*data));
+	if (!data) {
+		return -1;
+	}
+	memset(data, 0, sizeof(*data));
 
 	/*
 	 *	If the configuration parameters can't be parsed, then
@@ -378,6 +390,7 @@ static int counter_instantiate(CONF_SECTION *conf, void **instance)
 				data->counter_name);
 		return -1;
 	}
+	data->check_attr = dattr->attr;
 
 	/*
 	 * Find the attribute for the allowed protocol
@@ -441,7 +454,7 @@ static int counter_instantiate(CONF_SECTION *conf, void **instance)
 
 		memcpy(&next_reset, time_datum.dptr, sizeof(time_t));
 		free(time_datum.dptr);
-		if (next_reset <= now){
+		if (next_reset && next_reset <= now){
 
 			data->last_reset = now;
 			ret = reset_db(data);
@@ -489,11 +502,26 @@ static int counter_accounting(void *instance, REQUEST *request)
 	rlm_counter_t *data = (rlm_counter_t *)instance;
 	datum key_datum;
 	datum count_datum;
-	VALUE_PAIR *key_vp, *count_vp, *proto_vp;
-	int counter;
+	VALUE_PAIR *key_vp, *count_vp, *proto_vp, *uniqueid_vp;
+	rad_counter counter;
 	int rcode;
+	int acctstatustype = 0;
 	time_t diff;
 
+	if ((key_vp = pairfind(request->packet->vps, PW_ACCT_STATUS_TYPE)) != NULL)
+		acctstatustype = key_vp->lvalue;
+	else {
+		DEBUG("rlm_counter: Could not find account status type in packet.");
+		return RLM_MODULE_NOOP;
+	}
+	if (acctstatustype != PW_STATUS_STOP){
+		DEBUG("rlm_counter: We only run on Accounting-Stop packets.");
+		return RLM_MODULE_NOOP;
+	}
+	uniqueid_vp = pairfind(request->packet->vps, PW_ACCT_UNIQUE_SESSION_ID);
+	if (uniqueid_vp != NULL)
+		DEBUG("rlm_counter: Packet Unique ID = '%s'",uniqueid_vp->strvalue);
+
 	/*
 	 *	Before doing anything else, see if we have to reset
 	 *	the counters.
@@ -526,15 +554,19 @@ static int counter_accounting(void *instance, REQUEST *request)
 	 *	The REAL username, after stripping.
 	 */
 	key_vp = (data->key_attr == PW_USER_NAME) ? request->username : pairfind(request->packet->vps, data->key_attr);
-	if (key_vp == NULL)
+	if (key_vp == NULL){
+		DEBUG("rlm_counter: Could not find the key-attribute in the request.");
 		return RLM_MODULE_NOOP;
+	}
 
 	/*
 	 *	Look for the attribute to use as a counter.
 	 */
 	count_vp = pairfind(request->packet->vps, data->count_attr);
-	if (count_vp == NULL)
+	if (count_vp == NULL){
+		DEBUG("rlm_counter: Could not find the count-attribute in the request.");
 		return RLM_MODULE_NOOP;
+	}
 
 	key_datum.dptr = key_vp->strvalue;
 	key_datum.dsize = key_vp->length;
@@ -542,11 +574,27 @@ static int counter_accounting(void *instance, REQUEST *request)
 	pthread_mutex_lock(&data->mutex);
 	count_datum = gdbm_fetch(data->gdbm, key_datum);
 	pthread_mutex_unlock(&data->mutex);
-	if (count_datum.dptr == NULL)
-		counter = 0;
+	if (count_datum.dptr == NULL){
+		counter.user_counter = 0;
+		if (uniqueid_vp != NULL)
+			strncpy(uniqueid_vp->strvalue,counter.uniqueid,UNIQUEID_MAX_LEN - 1);
+		else
+			memset((char *)counter.uniqueid,0,UNIQUEID_MAX_LEN);
+	}
 	else{
-		memcpy(&counter, count_datum.dptr, sizeof(int));
+		memcpy(&counter, count_datum.dptr, sizeof(rad_counter));
 		free(count_datum.dptr);
+		if (counter.uniqueid)
+			DEBUG("rlm_counter: Counter Unique ID = '%s'",counter.uniqueid);
+		if (uniqueid_vp != NULL){ 
+			if (counter.uniqueid != NULL && 
+				strncmp(uniqueid_vp->strvalue,counter.uniqueid, UNIQUEID_MAX_LEN - 1) == 0){
+				DEBUG("rlm_counter: Unique IDs for user match. Droping the request.");
+				return RLM_MODULE_NOOP;
+			}
+			strncpy(counter.uniqueid,uniqueid_vp->strvalue,UNIQUEID_MAX_LEN - 1);
+		}
+		DEBUG("rlm_counter: User=%s, Counter=%d.",request->username->strvalue,counter.user_counter);
 	}
 
 	if (data->count_attr == PW_ACCT_SESSION_TIME) {
@@ -561,24 +609,26 @@ static int counter_accounting(void *instance, REQUEST *request)
 		 *	day). That is the right thing
 		 */
 		diff = request->timestamp - data->last_reset;
-		counter += (count_vp->lvalue < diff) ? count_vp->lvalue : diff;
+		counter.user_counter += (count_vp->lvalue < diff) ? count_vp->lvalue : diff;
 
 	} else if (count_vp->type == PW_TYPE_INTEGER) {
 		/*
 		 *	Integers get counted, without worrying about
 		 *	reset dates.
 		 */
-		counter += count_vp->lvalue;
+		counter.user_counter += count_vp->lvalue;
 
 	} else {
 		/*
 		 *	The attribute is NOT an integer, just count once
 		 *	more that we've seen it.
 		 */
-		counter++;
+		counter.user_counter++;
 	}
-	count_datum.dptr = (char *) &counter;
-	count_datum.dsize = sizeof(int);
+
+	DEBUG("rlm_counter: User=%s, New Counter=%d.",request->username->strvalue,counter.user_counter);
+	count_datum.dptr = (rad_counter *) &counter;
+	count_datum.dsize = sizeof(rad_counter);
 
 	pthread_mutex_lock(&data->mutex);
 	rcode = gdbm_store(data->gdbm, key_datum, count_datum, GDBM_REPLACE);
@@ -604,9 +654,8 @@ static int counter_authorize(void *instance, REQUEST *request)
 	int ret=RLM_MODULE_NOOP;
 	datum key_datum;
 	datum count_datum;
-	int counter=0;
+	rad_counter counter;
 	int res=0;
-	DICT_ATTR *dattr;
 	VALUE_PAIR *key_vp, *check_vp;
 	VALUE_PAIR *reply_item;
 	char msg[128];
@@ -646,29 +695,33 @@ static int counter_authorize(void *instance, REQUEST *request)
 	/*
 	 *      Look for the check item
 	 */
-	if ((dattr = dict_attrbyname(data->check_name)) == NULL) {
-		return ret;
-	}
-	if ((check_vp= pairfind(request->config_items, dattr->attr)) == NULL) {
+	if ((check_vp= pairfind(request->config_items, data->check_attr)) == NULL) {
 		DEBUG2("rlm_counter: Could not find Check item value pair");
 		return ret;
 	}
 
 	key_datum.dptr = key_vp->strvalue;
 	key_datum.dsize = key_vp->length;
+
+
+	/*
+	 * Init to be sure
+	 */
+
+	counter.user_counter = 0;
 	
 	pthread_mutex_lock(&data->mutex);
 	count_datum = gdbm_fetch(data->gdbm, key_datum);
 	pthread_mutex_unlock(&data->mutex);
 	if (count_datum.dptr != NULL){
-		memcpy(&counter, count_datum.dptr, sizeof(int));
+		memcpy(&counter, count_datum.dptr, sizeof(rad_counter));
 		free(count_datum.dptr);
 	}
 
 	/*
 	 * Check if check item > counter
 	 */
-	res=check_vp->lvalue - counter;
+	res=check_vp->lvalue - counter.user_counter;
 	if (res > 0) {
 		if (data->count_attr == PW_ACCT_SESSION_TIME) {
 			/*
@@ -715,7 +768,7 @@ static int counter_authorize(void *instance, REQUEST *request)
 
 		DEBUG2("rlm_counter: (Check item - counter) is greater than zero");
 		DEBUG2("rlm_counter: Authorized user %s, check_item=%d, counter=%d",
-				key_vp->strvalue,check_vp->lvalue,counter);
+				key_vp->strvalue,check_vp->lvalue,counter.user_counter);
 		DEBUG2("rlm_counter: Sent Reply-Item for user %s, Type=Session-Timeout, value=%d",
 				key_vp->strvalue,res);
 	}
@@ -737,7 +790,7 @@ static int counter_authorize(void *instance, REQUEST *request)
 		ret=RLM_MODULE_REJECT;
 
 		DEBUG2("rlm_counter: Rejected user %s, check_item=%d, counter=%d",
-				key_vp->strvalue,check_vp->lvalue,counter);
+				key_vp->strvalue,check_vp->lvalue,counter.user_counter);
 	}
 
 	return ret;
