@@ -103,8 +103,22 @@
  *	- Use the MAX_FAILED_CONNS_* in ldap_authenticate() when calling ldap_connect()
  * Nov 2002, Kostas Kalevras <kkalev@noc.ntua.gr>
  *	- Set LDAP version to V3 before binding. Now freeradius should work with openldap21
+ * Dec 2002, Kostas Kalevras <kkalev@noc.ntua.gr>
+ *	- Set default values for the server and basedn parameters
+ * Feb 2003, Kostas Kalevras <kkalev@noc.ntua.gr>
+ *	- Add support for ldap_initialize. That way we can specify the server as an ldap url.
+ *	  Based on ideas from Derrik Pates <dpates@dsdk12.net>
+ * Mar 2003, Kostas Kalevras <kkalev@noc.ntua.gr>
+ * 	- Add an ldap_escape_func. Escape the * character from the filter so that we can avoid
+ * 	  the trivial DoS of username=*
+ * 	- Remove the caching code. It does not exist in openldap21.
+ *	  Based on a report from Mike Denka <mdenk@whidbey.net>
+ * May 2003, Kostas Kalevras <kkalev@noc.ntua.gr>
+ *	- Don't do a double free on the attribute maps. Bug noted by Derrik Pates <dpates@dsdk12.net>
+ *	- Apply a patch from Alexander M. Pravking <fduch@antar.bryansk.ru> to do an xlat on the
+ *	  retrieved attributes.
  */
-static const char rcsid[] = "$Id: rlm_ldap.c,v 1.96 2002/11/01 21:36:43 kkalev Exp $";
+static const char rcsid[] = "$Id: rlm_ldap.c,v 1.108 2003/07/07 19:07:08 aland Exp $";
 
 #include "autoconf.h"
 
@@ -134,9 +148,7 @@ static const char rcsid[] = "$Id: rlm_ldap.c,v 1.96 2002/11/01 21:36:43 kkalev E
 #include	"modules.h"
 #include	"rad_assert.h"
 
-
-#define MAX_AUTH_QUERY_LEN      256
-#define MAX_GROUP_STR_LEN	1024
+#define MAX_FILTER_STR_LEN	1024
 #define TIMELIMIT 5
 
 /*
@@ -184,8 +196,6 @@ typedef struct ldap_conn {
 	pthread_mutex_t	mutex;
 } LDAP_CONN;
 
-#define MAX_SERVER_LINE 1024
-
 typedef struct {
 	char           *server;
 	int             port;
@@ -196,11 +206,10 @@ typedef struct {
 	int             tls_mode;
 	int		start_tls;
 	int		num_conns;
-	int		cache_timeout;
-	int		cache_size;
 	int		do_comp;
 	int		default_allow;
 	int		failed_conns;
+	int		is_url;
 	char           *login;
 	char           *password;
 	char           *filter;
@@ -223,7 +232,7 @@ typedef struct {
 }               ldap_instance;
 
 static CONF_PARSER module_config[] = {
-	{"server", PW_TYPE_STRING_PTR, offsetof(ldap_instance,server), NULL, NULL},
+	{"server", PW_TYPE_STRING_PTR, offsetof(ldap_instance,server), NULL, "localhost"},
 	{"port", PW_TYPE_INTEGER, offsetof(ldap_instance,port), NULL, "389"},
 	/* wait forever on network activity */
 	{"net_timeout", PW_TYPE_INTEGER, offsetof(ldap_instance,net_timeout.tv_sec), NULL, "10"},
@@ -231,13 +240,10 @@ static CONF_PARSER module_config[] = {
 	{"timeout", PW_TYPE_INTEGER, offsetof(ldap_instance,timeout.tv_sec), NULL, "20"},
 	/* allow server unlimited time for search (server-side limit) */
 	{"timelimit", PW_TYPE_INTEGER, offsetof(ldap_instance,timelimit), NULL, "20"},
-	{"ldap_cache_timeout", PW_TYPE_INTEGER, offsetof(ldap_instance,cache_timeout), NULL, "0"},
-	{"ldap_cache_size", PW_TYPE_INTEGER, offsetof(ldap_instance,cache_size), NULL, "0"},
 	{"identity", PW_TYPE_STRING_PTR, offsetof(ldap_instance,login), NULL, ""},
 	{"start_tls", PW_TYPE_BOOLEAN, offsetof(ldap_instance,start_tls), NULL, "no"},
-	{"tls_mode", PW_TYPE_BOOLEAN, offsetof(ldap_instance,tls_mode), NULL, "no"},
 	{"password", PW_TYPE_STRING_PTR, offsetof(ldap_instance,password), NULL, ""},
-	{"basedn", PW_TYPE_STRING_PTR, offsetof(ldap_instance,basedn), NULL, NULL},
+	{"basedn", PW_TYPE_STRING_PTR, offsetof(ldap_instance,basedn), NULL, "o=notexist"},
 	{"filter", PW_TYPE_STRING_PTR, offsetof(ldap_instance,filter), NULL, "(uid=%u)"},
 	{"default_profile", PW_TYPE_STRING_PTR, offsetof(ldap_instance,default_profile), NULL, NULL},
 	{"profile_attribute", PW_TYPE_STRING_PTR, offsetof(ldap_instance,profile_attr), NULL, NULL},
@@ -316,6 +322,10 @@ ldap_instantiate(CONF_SECTION * conf, void **instance)
 	char *xlat_name;
 
 	inst = rad_malloc(sizeof *inst);
+	if (!inst) {
+		return -1;
+	}
+	memset(inst, 0, sizeof(*inst));
 
 	if (cf_section_parse(conf, inst, module_config) < 0) {
 		free(inst);
@@ -327,14 +337,21 @@ ldap_instantiate(CONF_SECTION * conf, void **instance)
 		free(inst);
 		return -1;
 	}
+	inst->is_url = 0;
+#ifdef HAVE_LDAP_INITIALIZE
+	if (ldap_is_ldap_url(inst->server)){
+		inst->is_url = 1;
+		inst->port = 0;
+	}
+#endif
  
 	inst->timeout.tv_usec = 0;
 	inst->net_timeout.tv_usec = 0;
 	/* workaround for servers which support LDAPS but not START TLS */
 	if(inst->port == LDAPS_PORT)
 		inst->tls_mode = LDAP_OPT_X_TLS_HARD;
-	else if (inst->tls_mode)
-  		inst->tls_mode = LDAP_OPT_X_TLS_TRY;
+	else
+		inst->tls_mode = 0;
 	inst->reply_item_map = NULL;
 	inst->check_item_map = NULL;
 	inst->conns = NULL;
@@ -564,8 +581,6 @@ retry:
 		DEBUG2("rlm_ldap: attempting LDAP reconnection");
 		if (conn->ld){
 			DEBUG2("rlm_ldap: closing existing LDAP connection");
-			if (inst->cache_timeout >0)
-				ldap_destroy_cache(conn->ld);
 			ldap_unbind_s(conn->ld);
 		}
 		if ((conn->ld = ldap_connect(instance, inst->login, inst->password, 0, &res)) == NULL) {
@@ -629,6 +644,40 @@ retry:
 }
 
 
+/*
+ *	Translate the LDAP queries.
+ */
+static int ldap_escape_func(char *out, int outlen, const char *in)
+{
+	int len = 0;
+	
+	while (in[0]) {
+		/*
+		 *  Only one byte left.
+		 */
+		if (outlen <= 1) {
+			break;
+		}
+		
+		if (strchr("*", *in)) {
+			in++;
+			outlen--;
+			continue;
+		}
+		
+		/*
+		 *	Else it's a nice character.
+		 */
+		*out = *in;
+		out++;
+		in++;
+		outlen--;
+		len++;
+	}
+	*out = '\0';
+	return len;
+}
+
 /*
  * ldap_groupcmp(). Implement the Ldap-Group == "group" filter
  */
@@ -636,23 +685,26 @@ retry:
 static int ldap_groupcmp(void *instance, REQUEST *req, VALUE_PAIR *request, VALUE_PAIR *check,
                 VALUE_PAIR *check_pairs, VALUE_PAIR **reply_pairs)
 {
-        char            filter[MAX_GROUP_STR_LEN];
-        char            gr_filter[MAX_AUTH_QUERY_LEN];
+        char            filter[MAX_FILTER_STR_LEN];
+        char            gr_filter[MAX_FILTER_STR_LEN];
         int             res;
         LDAPMessage     *result = NULL;
         LDAPMessage     *msg = NULL;
-        char            basedn[1024];
+        char            basedn[MAX_FILTER_STR_LEN];
 	char		*attrs[] = {"dn",NULL};
+	char		**vals;
         ldap_instance   *inst = instance;
+	char		*group_attrs[] = {inst->groupmemb_attr,NULL};
 	LDAP_CONN	*conn;
 	int		conn_id = -1;
+	VALUE_PAIR	*vp_user_dn;
+	VALUE_PAIR      **request_pairs;
 
-        check_pairs = check_pairs;
-        reply_pairs = reply_pairs;
+	request_pairs = &req->packet->vps;
 
 	DEBUG("rlm_ldap: Entering ldap_groupcmp()");
 
-        if (check->strvalue == NULL || check->length == 0){
+	if (check->strvalue == NULL || check->length == 0){
                 DEBUG("rlm_ldap::ldap_groupcmp: Illegal group name");
                 return 1;
         }
@@ -667,10 +719,11 @@ static int ldap_groupcmp(void *instance, REQUEST *req, VALUE_PAIR *request, VALU
                 return 1;
         }
 
-        if ((pairfind(req->packet->vps, PW_LDAP_USERDN)) == NULL){
+        while((vp_user_dn = pairfind(*request_pairs, PW_LDAP_USERDN)) == NULL){
                 char            *user_dn = NULL;
 
-                if (!radius_xlat(filter, MAX_AUTH_QUERY_LEN, inst->filter, req, NULL)) {
+                if (!radius_xlat(filter, sizeof(filter), inst->filter,
+					req, ldap_escape_func)){
                         DEBUG("rlm_ldap::ldap_groupcmp: unable to create filter");
                         return 1;
                 }
@@ -678,7 +731,8 @@ static int ldap_groupcmp(void *instance, REQUEST *req, VALUE_PAIR *request, VALU
 			radlog(L_ERR, "rlm_ldap: All ldap connections are in use");
 			return 1;
 		}
-                if ((res = perform_search(inst, conn, basedn, LDAP_SCOPE_SUBTREE, filter, attrs, &result)) != RLM_MODULE_OK) {
+                if ((res = perform_search(inst, conn, basedn, LDAP_SCOPE_SUBTREE,
+					filter, attrs, &result)) != RLM_MODULE_OK){
                         DEBUG("rlm_ldap::ldap_groupcmp: search failed");
 			ldap_release_conn(conn_id,inst->conns);
                         return 1;
@@ -700,115 +754,120 @@ static int ldap_groupcmp(void *instance, REQUEST *req, VALUE_PAIR *request, VALU
                 * Adding new attribute containing DN for LDAP object associated with
                 * given username
                 */
-                pairadd(&req->packet->vps, pairmake("Ldap-UserDn", user_dn, T_OP_EQ));
+                pairadd(request_pairs, pairmake("Ldap-UserDn", user_dn, T_OP_EQ));
                 ldap_memfree(user_dn);
                 ldap_msgfree(result);
         }
 
-        if(!radius_xlat(gr_filter, MAX_GROUP_STR_LEN, inst->groupmemb_filt, req, NULL)){
+        if(!radius_xlat(gr_filter, sizeof(gr_filter), inst->groupmemb_filt, req, NULL)){
                 DEBUG("rlm_ldap::ldap_groupcmp: unable to create filter.");
                 return 1;
         }
 
-	snprintf(filter,MAX_GROUP_STR_LEN - 1, "(&(%s=%s)%s)",inst->groupname_attr,(char *)check->strvalue,gr_filter);
+	if (strchr((char *)check->strvalue,',') != NULL) {
+		/* This looks like a DN */
+		snprintf(filter,sizeof(filter), "%s",gr_filter);
+		snprintf(basedn,sizeof(basedn), "%s",(char *)check->strvalue);
+	} else 
+		snprintf(filter,sizeof(filter), "(&(%s=%s)%s)",inst->groupname_attr,(char *)check->strvalue,gr_filter);
 
 	if ((conn_id = ldap_get_conn(inst->conns,&conn,inst)) == -1){
 		radlog(L_ERR, "rlm_ldap: All ldap connections are in use");
 		return 1;
 	}
-
-        if ((res = perform_search(inst, conn, basedn, LDAP_SCOPE_SUBTREE, filter, attrs, &result)) != RLM_MODULE_OK){
-                if (res != RLM_MODULE_NOTFOUND){
-                	DEBUG("rlm_ldap::ldap_groupcmp: Search returned error");
-			ldap_release_conn(conn_id,inst->conns);
-                	return 1;
-                }
-        }
-	if (res != RLM_MODULE_NOTFOUND)
+	
+	if ((res = perform_search(inst, conn, basedn, LDAP_SCOPE_SUBTREE,
+				filter, attrs, &result)) == RLM_MODULE_OK){
+		DEBUG("rlm_ldap::ldap_groupcmp: User found in group %s",
+				(char *)check->strvalue);
 		ldap_msgfree(result);
+		ldap_release_conn(conn_id,inst->conns);
+        	return 0;
+	}
+	
 	ldap_release_conn(conn_id,inst->conns);
 
-	if (res == RLM_MODULE_NOTFOUND){
-		if (inst->groupmemb_attr == NULL){
-			DEBUG("rlm_ldap::ldap_groupcmp: Group %s not found or user is not a member.",(char *)check->strvalue);
-			return 1;
-		}
-		else{
-			VALUE_PAIR *user_dn;
-			char *group_attrs[] = {inst->groupmemb_attr,NULL};
+	if (res != RLM_MODULE_NOTFOUND ) {
+		DEBUG("rlm_ldap::ldap_groupcmp: Search returned error");
+		return 1;
+	}
 
-			user_dn = pairfind(req->packet->vps, PW_LDAP_USERDN);
-			if (user_dn != NULL){
-				char **vals;
+	if (inst->groupmemb_attr == NULL){
+		/* search returned NOTFOUND and searching for membership 
+		 * using user object attributes is not specified in config
+		 * file
+		 */
+		DEBUG("rlm_ldap::ldap_groupcmp: Group %s not found or user is not a member.",(char *)check->strvalue);
+		return 1;
+	}
 
-				snprintf(filter,MAX_GROUP_STR_LEN - 1, "(objectclass=*)");
-				if ((conn_id = ldap_get_conn(inst->conns,&conn,inst)) == -1){
-					radlog(L_ERR, "rlm_ldap: Add ldap connections are in use");
-					return 1;
-				}
-				if ((res = perform_search(inst, conn, user_dn->strvalue, LDAP_SCOPE_BASE, filter, group_attrs, &result)) != RLM_MODULE_OK){
-					DEBUG("rlm_ldap::ldap_groupcmp: Search returned error");
-					ldap_release_conn(conn_id, inst->conns);
-					return 1;
-				}		
+	snprintf(filter,sizeof(filter), "(objectclass=*)");
+	if ((conn_id = ldap_get_conn(inst->conns,&conn,inst)) == -1){
+		radlog(L_ERR, "rlm_ldap: Add ldap connections are in use");
+		return 1;
+	}
+	if ((res = perform_search(inst, conn, vp_user_dn->strvalue, LDAP_SCOPE_BASE,
+					filter, group_attrs,&result)) != RLM_MODULE_OK){
+		DEBUG("rlm_ldap::ldap_groupcmp: Search returned error");
+		ldap_release_conn(conn_id, inst->conns);
+		return 1;
+	}		
 
-				if ((msg = ldap_first_entry(conn->ld, result)) == NULL) {
-					DEBUG("rlm_ldap::ldap_groupcmp: ldap_first_entry() failed");
-					ldap_release_conn(conn_id,inst->conns);
-					ldap_msgfree(result);
-					return 1;
-				}
-				if ((vals = ldap_get_values(conn->ld, msg, inst->groupmemb_attr)) != NULL) {
-					unsigned int i = 0;
-					char found = 0;
-
-					for (;i < ldap_count_values(vals);i++){
-						if (strchr(vals[i],',') != NULL){ /* This looks like a DN */
-							LDAPMessage *gr_result = NULL;
-
-							snprintf(filter,MAX_GROUP_STR_LEN - 1, "(%s=%s)",inst->groupname_attr,(char *)check->strvalue);
-							if ((res = perform_search(inst, conn, vals[i], LDAP_SCOPE_BASE, filter, attrs, &gr_result)) != RLM_MODULE_OK){
-								if (res != RLM_MODULE_NOTFOUND){
-									DEBUG("rlm_ldap::ldap_groupcmp: Search returned error");
-									ldap_msgfree(result);
-									ldap_value_free(vals);
-									ldap_release_conn(conn_id, inst->conns);
-									return 1;
-								}
-							}
-							else{
-								ldap_msgfree(gr_result);
-								found = 1;
-								break;
-							}
-						}
-						else{
-							if (strcmp(vals[i],(char *)check->strvalue) == 0){
-								found = 1;
-								break;
-							}
-						}
-					}
-					ldap_value_free(vals);
-        				ldap_msgfree(result);
-					if (found == 0){
-						DEBUG("rlm_ldap::groupcmp: Group %s not found or user not a member",
-							(char *)check->strvalue);
+	if ((msg = ldap_first_entry(conn->ld, result)) == NULL) {
+		DEBUG("rlm_ldap::ldap_groupcmp: ldap_first_entry() failed");
+		ldap_release_conn(conn_id,inst->conns);
+		ldap_msgfree(result);
+		return 1;
+	}
+	if ((vals = ldap_get_values(conn->ld, msg, inst->groupmemb_attr)) != NULL) {
+		unsigned int i = 0;
+		char found = 0;
+		for (;i < ldap_count_values(vals);i++){
+			if (strchr(vals[i],',') != NULL){ 
+				/* This looks like a DN */
+				LDAPMessage *gr_result = NULL;
+				snprintf(filter,sizeof(filter), "(%s=%s)",
+					inst->groupname_attr,
+					(char *)check->strvalue);
+				if ((res = perform_search(inst, conn, vals[i],
+						LDAP_SCOPE_BASE, filter,
+						attrs, &gr_result)) != RLM_MODULE_OK){
+					if (res != RLM_MODULE_NOTFOUND){
+						DEBUG("rlm_ldap::ldap_groupcmp: \
+							Search returned error");
+						ldap_value_free(vals);
+						ldap_msgfree(result);
 						ldap_release_conn(conn_id,inst->conns);
 						return 1;
 					}
+				} else {
+					ldap_msgfree(gr_result);
+					found = 1;
+					break;
 				}
-				else{
-					DEBUG("rlm_ldap::ldap_groupcmp: ldap_get_values() failed");
-					ldap_release_conn(conn_id,inst->conns);
-					ldap_msgfree(result);
-					return 1;
+			} else {
+				if (strcmp(vals[i],(char *)check->strvalue) == 0){
+					found = 1;
+					break;
 				}
 			}
 		}
+		ldap_value_free(vals);
+		ldap_msgfree(result);
+		if (found == 0){
+			DEBUG("rlm_ldap::groupcmp: Group %s not found \
+				or user not a member",
+				(char *)check->strvalue);
+			ldap_release_conn(conn_id,inst->conns);
+			return 1;
+		}
+	} else {
+			DEBUG("rlm_ldap::ldap_groupcmp: ldap_get_values() failed");
+			ldap_msgfree(result);
+			ldap_release_conn(conn_id,inst->conns);
+			return 1;
 	}
 
-
 	DEBUG("rlm_ldap::ldap_groupcmp: User found in group %s",(char *)check->strvalue);
 	ldap_release_conn(conn_id,inst->conns);
 
@@ -823,7 +882,7 @@ static int ldap_groupcmp(void *instance, REQUEST *req, VALUE_PAIR *request, VALU
 static int ldap_xlat(void *instance, REQUEST *request, char *fmt, char *out, int freespace,
 				RADIUS_ESCAPE_STRING func)
 {
-	char url[MAX_STRING_LEN];
+	char url[MAX_FILTER_STR_LEN];
 	int res;
 	int ret = 0;
 	ldap_instance *inst = instance;
@@ -931,8 +990,8 @@ ldap_authorize(void *instance, REQUEST * request)
 	LDAPMessage	*def_attr_result = NULL;
 	ldap_instance	*inst = instance;
 	char		*user_dn = NULL;
-	char		filter[MAX_AUTH_QUERY_LEN];
-	char		basedn[1024];
+	char		filter[MAX_FILTER_STR_LEN];
+	char		basedn[MAX_FILTER_STR_LEN];
 	VALUE_PAIR	*check_tmp;
 	VALUE_PAIR	*reply_tmp;
 	int		res;
@@ -965,7 +1024,7 @@ ldap_authorize(void *instance, REQUEST * request)
 	       request->username->strvalue);
 
 	if (!radius_xlat(filter, sizeof(filter), inst->filter,
-			 request, NULL)) {
+			 request, ldap_escape_func)) {
 		radlog (L_ERR, "rlm_ldap: unable to create filter.\n");
 		return RLM_MODULE_INVALID;
 	}
@@ -1049,8 +1108,6 @@ ldap_authorize(void *instance, REQUEST * request)
 			}
 		}
 	}
-	if (inst->cache_timeout >0)
-		ldap_enable_cache(conn->ld, inst->cache_timeout, inst->cache_size);
 
 	/*
 	 * Check for the default profile entry. If it exists then add the 
@@ -1061,7 +1118,7 @@ ldap_authorize(void *instance, REQUEST * request)
 	if (inst->default_profile || user_profile){
 		char *profile = inst->default_profile;
 
-		strncpy(filter,"(objectclass=radiusprofile)",MAX_AUTH_QUERY_LEN);
+		strNcpy(filter,"(objectclass=radiusprofile)",sizeof(filter));
 		if (user_profile)
 			profile = user_profile->strvalue;
 		if (profile && strlen(profile)){
@@ -1069,18 +1126,22 @@ ldap_authorize(void *instance, REQUEST * request)
 				profile, LDAP_SCOPE_BASE, 
 				filter, inst->atts, &def_result)) == RLM_MODULE_OK){
 				if ((def_msg = ldap_first_entry(conn->ld,def_result))){
-					if ((check_tmp = ldap_pairget(conn->ld,def_msg,inst->check_item_map,check_pairs,1)))
-						pairadd(check_pairs,check_tmp);
-					if ((reply_tmp = ldap_pairget(conn->ld,def_msg,inst->reply_item_map,reply_pairs,0)))
-						pairadd(reply_pairs,reply_tmp);
+					if ((check_tmp = ldap_pairget(conn->ld,def_msg,inst->check_item_map,check_pairs,1))) {
+						/*pairadd(check_pairs,check_tmp);*/
+						pairxlatmove(request, check_pairs, &check_tmp);
+						pairfree(&check_tmp);
+					}
+					if ((reply_tmp = ldap_pairget(conn->ld,def_msg,inst->reply_item_map,reply_pairs,0))) {
+						/*pairadd(reply_pairs,reply_tmp);*/
+						pairxlatmove(request, reply_pairs, &reply_tmp);
+						pairfree(&reply_tmp);
+					}
 				}
 				ldap_msgfree(def_result);
 			} else 
 				DEBUG("rlm_ldap: default_profile/user-profile search failed");
 		}
 	}
-	if (inst->cache_timeout >0)
-		ldap_disable_cache(conn->ld);
 
 	/*
 	 * Check for the profile attribute. If it exists, we assume that it 
@@ -1092,18 +1153,22 @@ ldap_authorize(void *instance, REQUEST * request)
 	if (inst->profile_attr){
 		if ((vals = ldap_get_values(conn->ld, msg, inst->profile_attr)) != NULL) {
 			unsigned int i=0;
-			strncpy(filter,"(objectclass=radiusprofile)",MAX_AUTH_QUERY_LEN);
-			if (inst->cache_timeout >0)
-				ldap_enable_cache(conn->ld, inst->cache_timeout, inst->cache_size);
+			strNcpy(filter,"(objectclass=radiusprofile)",sizeof(filter));
 			while(vals[i] != NULL && strlen(vals[i])){
 				if ((res = perform_search(instance, conn,
 					vals[i], LDAP_SCOPE_BASE, 
 					filter, inst->atts, &def_attr_result)) == RLM_MODULE_OK){
 					if ((def_attr_msg = ldap_first_entry(conn->ld,def_attr_result))){
-						if ((check_tmp = ldap_pairget(conn->ld,def_attr_msg,inst->check_item_map,check_pairs,1)))
-							pairadd(check_pairs,check_tmp);
-						if ((reply_tmp = ldap_pairget(conn->ld,def_attr_msg,inst->reply_item_map,reply_pairs,0)))
-							pairadd(reply_pairs,reply_tmp);
+						if ((check_tmp = ldap_pairget(conn->ld,def_attr_msg,inst->check_item_map,check_pairs,1))) {
+							/*pairadd(check_pairs,check_tmp);*/
+							pairxlatmove(request, check_pairs, &check_tmp);
+							pairfree(&check_tmp);
+						}
+						if ((reply_tmp = ldap_pairget(conn->ld,def_attr_msg,inst->reply_item_map,reply_pairs,0))) {
+							/*pairadd(reply_pairs,reply_tmp);*/
+							pairxlatmove(request, reply_pairs, &reply_tmp);
+							pairfree(&reply_tmp);
+						}
 					}
 					ldap_msgfree(def_attr_result);
 				} else 
@@ -1113,8 +1178,6 @@ ldap_authorize(void *instance, REQUEST * request)
 			ldap_value_free(vals);
 		}
 	}
-	if (inst->cache_timeout >0)
-		ldap_disable_cache(conn->ld);
 	if (inst->passwd_attr && strlen(inst->passwd_attr)){
 		VALUE_PAIR *passwd_item;
 
@@ -1162,15 +1225,21 @@ ldap_authorize(void *instance, REQUEST * request)
 
 	DEBUG("rlm_ldap: looking for check items in directory...");
 
-	if ((check_tmp = ldap_pairget(conn->ld, msg, inst->check_item_map,check_pairs,1)) != NULL)
-		pairadd(check_pairs, check_tmp);
+	if ((check_tmp = ldap_pairget(conn->ld, msg, inst->check_item_map,check_pairs,1)) != NULL) {
+		/*pairadd(check_pairs,check_tmp);*/
+		pairxlatmove(request, check_pairs, &check_tmp);
+		pairfree(&check_tmp);
+	}
 
 
 	DEBUG("rlm_ldap: looking for reply items in directory...");
 
 
-	if ((reply_tmp = ldap_pairget(conn->ld, msg, inst->reply_item_map,reply_pairs,0)) != NULL)
-		pairadd(reply_pairs, reply_tmp);
+	if ((reply_tmp = ldap_pairget(conn->ld, msg, inst->reply_item_map,reply_pairs,0)) != NULL) {
+		/*pairadd(reply_pairs,reply_tmp);*/
+		pairxlatmove(request, reply_pairs, &reply_tmp);
+		pairfree(&reply_tmp);
+	}
 
        if (inst->do_comp && paircmp(request,request->packet->vps,*check_pairs,reply_pairs) != 0){
 		DEBUG("rlm_ldap: Pairs do not match. Rejecting user.");
@@ -1213,8 +1282,8 @@ ldap_authenticate(void *instance, REQUEST * request)
 	LDAPMessage    *result, *msg;
 	ldap_instance  *inst = instance;
 	char           *user_dn, *attrs[] = {"uid", NULL};
-        char		filter[MAX_AUTH_QUERY_LEN];
-	char		basedn[1024];
+	char		filter[MAX_FILTER_STR_LEN];
+	char		basedn[MAX_FILTER_STR_LEN];
 	int             res;
 	VALUE_PAIR     *vp_user_dn;
 	VALUE_PAIR      *module_fmsg_vp;
@@ -1339,16 +1408,28 @@ static LDAP    *
 ldap_connect(void *instance, const char *dn, const char *password, int auth, int *result)
 {
 	ldap_instance  *inst = instance;
-	LDAP           *ld;
+	LDAP           *ld = NULL;
 	int             msgid, rc, ldap_version;
 	int		ldap_errno = 0;
 	LDAPMessage    *res;
 
-	DEBUG("rlm_ldap: (re)connect to %s:%d, authentication %d", inst->server, inst->port, auth);
-	if ((ld = ldap_init(inst->server, inst->port)) == NULL) {
-		radlog(L_ERR, "rlm_ldap: ldap_init() failed");
-		*result = RLM_MODULE_FAIL;
-		return (NULL);
+	if (inst->is_url){
+#ifdef HAVE_LDAP_INITIALIZE
+		DEBUG("rlm_ldap: (re)connect to %s, authentication %d", inst->server, auth);
+		if (ldap_initialize(&ld, inst->server) != LDAP_SUCCESS) {
+			radlog(L_ERR, "rlm_ldap: ldap_initialize() failed");
+			*result = RLM_MODULE_FAIL;
+			return (NULL);
+		}
+#endif
+	}
+	else{
+		DEBUG("rlm_ldap: (re)connect to %s:%d, authentication %d", inst->server, inst->port, auth);
+		if ((ld = ldap_init(inst->server, inst->port)) == NULL) {
+			radlog(L_ERR, "rlm_ldap: ldap_init() failed");
+			*result = RLM_MODULE_FAIL;
+			return (NULL);
+		}
 	}
 	if (ldap_set_option(ld, LDAP_OPT_NETWORK_TIMEOUT, (void *) &(inst->net_timeout)) != LDAP_OPT_SUCCESS) {
 		radlog(L_ERR, "rlm_ldap: Could not set LDAP_OPT_NETWORK_TIMEOUT %ld.%ld", inst->net_timeout.tv_sec, inst->net_timeout.tv_usec);
@@ -1372,7 +1453,6 @@ ldap_connect(void *instance, const char *dn, const char *password, int auth, int
  			radlog(L_ERR, "rlm_ldap: could not set LDAP_OPT_X_TLS option %s", ldap_err2string(ldap_errno));
 		}
 	}
-
 	if (inst->start_tls) {
 		DEBUG("rlm_ldap: starting TLS");
 		rc = ldap_start_tls_s(ld, NULL, NULL);
@@ -1387,13 +1467,22 @@ ldap_connect(void *instance, const char *dn, const char *password, int auth, int
 	}
 #endif /* HAVE_LDAP_START_TLS */
 
-	DEBUG("rlm_ldap: bind as %s/%s to %s:%d", dn, password, inst->server, inst->port);
+	if (inst->is_url){
+		DEBUG("rlm_ldap: bind as %s/%s to %s", dn, password, inst->server);
+	}
+	else{
+		DEBUG("rlm_ldap: bind as %s/%s to %s:%d", dn, password, inst->server, inst->port);
+	}
 	msgid = ldap_bind(ld, dn, password,LDAP_AUTH_SIMPLE);
 	if (msgid == -1) {
 		ldap_get_option(ld, LDAP_OPT_ERROR_NUMBER, &ldap_errno);
-		radlog(L_ERR, "rlm_ldap: %s bind to %s:%d failed: %s",
-			 dn, inst->server, inst->port,
-			 ldap_err2string(ldap_errno));
+		if (inst->is_url)
+			radlog(L_ERR, "rlm_ldap: %s bind to %s failed: %s",
+			 	dn, inst->server, ldap_err2string(ldap_errno));
+		else
+			radlog(L_ERR, "rlm_ldap: %s bind to %s:%d failed: %s",
+			 	dn, inst->server, inst->port,
+			 	ldap_err2string(ldap_errno));
 		*result = RLM_MODULE_FAIL;
 		ldap_unbind_s(ld);
 		return (NULL);
@@ -1405,9 +1494,13 @@ ldap_connect(void *instance, const char *dn, const char *password, int auth, int
 	if(rc < 1) {
 		DEBUG("rlm_ldap: ldap_result()");
 		ldap_get_option(ld, LDAP_OPT_ERROR_NUMBER, &ldap_errno);
-		radlog(L_ERR, "rlm_ldap: %s bind to %s:%d failed: %s",
-			dn, inst->server, inst->port,
-			(rc == 0) ? "timeout" : ldap_err2string(ldap_errno));
+		if (inst->is_url)
+			radlog(L_ERR, "rlm_ldap: %s bind to %s failed: %s",
+				dn, inst->server, (rc == 0) ? "timeout" : ldap_err2string(ldap_errno));
+		else
+			radlog(L_ERR, "rlm_ldap: %s bind to %s:%d failed: %s",
+				dn, inst->server, inst->port,
+				(rc == 0) ? "timeout" : ldap_err2string(ldap_errno));
 		*result = RLM_MODULE_FAIL;
 		ldap_unbind_s(ld);
 		return (NULL);
@@ -1428,9 +1521,13 @@ ldap_connect(void *instance, const char *dn, const char *password, int auth, int
 		break;
 		
 	default:
-		radlog(L_ERR,"rlm_ldap: %s bind to %s:%d failed %s",
-			dn, inst->server, inst->port,
-			ldap_err2string(ldap_errno));
+		if (inst->is_url)
+			radlog(L_ERR,"rlm_ldap: %s bind to %s failed %s",
+				dn, inst->server, ldap_err2string(ldap_errno));
+		else
+			radlog(L_ERR,"rlm_ldap: %s bind to %s:%d failed %s",
+				dn, inst->server, inst->port,
+				ldap_err2string(ldap_errno));
 		*result = RLM_MODULE_FAIL;
 	}
 
@@ -1474,13 +1571,15 @@ ldap_detach(void *instance)
 		free((char *) inst->groupmemb_filt);
 	if (inst->groupmemb_attr)
 		free((char *) inst->groupmemb_attr);
+	if (inst->access_attr)
+		free((char *) inst->access_attr);
+	if (inst->profile_attr)
+		free((char *) inst->profile_attr);
 	if (inst->conns){
 		int i=0;
 
 		for(;i<inst->num_conns;i++){
 			if (inst->conns[i].ld){
-				if (inst->cache_timeout >0)
-					ldap_destroy_cache(inst->conns[i].ld);
 				ldap_unbind_s(inst->conns[i].ld);
 			}
 			pthread_mutex_destroy(&inst->conns[i].mutex);
@@ -1508,13 +1607,8 @@ ldap_detach(void *instance)
 		pair = nextpair;
 	}
 
-	if (inst->atts){
-		int i = 0;
-
-		while(inst->atts[i])
-			free(inst->atts[i++]);
+	if (inst->atts)
 		free(inst->atts);
-	}
 
 	paircompare_unregister(PW_LDAP_GROUP, ldap_groupcmp);
 	xlat_unregister(inst->xlat_name,ldap_xlat);
