@@ -16,7 +16,7 @@
  * 		    Reply-Message = "NS-MTA-MD5 is working!"
  * 		password = "testpass".
  *
- *  Version:	$Id: rlm_ns_mta_md5.c,v 1.7 2002/01/13 11:12:05 fcusack Exp $
+ *  Version:	$Id: rlm_ns_mta_md5.c,v 1.8 2002/08/24 16:54:59 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -37,7 +37,7 @@
  */
 
 
-static const char rcsid[] = "$Id: rlm_ns_mta_md5.c,v 1.7 2002/01/13 11:12:05 fcusack Exp $";
+static const char rcsid[] = "$Id: rlm_ns_mta_md5.c,v 1.8 2002/08/24 16:54:59 aland Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -170,7 +170,10 @@ module_t rlm_ns_mta_md5 = {
 	  NULL,			/* authorize */
 	  NULL,			/* pre-accounting */
 	  NULL,			/* accounting */
-	  NULL			/* checksimul */
+	  NULL,			/* checksimul */
+	  NULL,			/* pre-proxy */
+	  NULL,			/* post-proxy */
+	  NULL			/* post-auth */
   },
   NULL,				/* detach */
   NULL,				/* destroy */
