@@ -27,10 +27,10 @@ CREATE TABLE radacct (
   AcctAuthentic varchar(32) default NULL,
   ConnectInfo_start varchar(32) default NULL,
   ConnectInfo_stop varchar(32) default NULL,
-  AcctInputOctets int(12) default NULL,
-  AcctOutputOctets int(12) default NULL,
-  CalledStationId varchar(10) NOT NULL default '',
-  CallingStationId varchar(10) NOT NULL default '',
+  AcctInputOctets bigint(12) default NULL,
+  AcctOutputOctets bigint(12) default NULL,
+  CalledStationId varchar(50) NOT NULL default '',
+  CallingStationId varchar(50) NOT NULL default '',
   AcctTerminateCause varchar(32) NOT NULL default '',
   ServiceType varchar(32) default NULL,
   FramedProtocol varchar(32) default NULL,
@@ -55,7 +55,7 @@ CREATE TABLE radcheck (
   id int(11) unsigned NOT NULL auto_increment,
   UserName varchar(64) NOT NULL default '',
   Attribute varchar(32)  NOT NULL default '',
-  op char(2) NOT NULL,
+  op char(2) NOT NULL DEFAULT '==',
   Value varchar(253) NOT NULL default '',
   PRIMARY KEY  (id),
   KEY UserName (UserName(32))
@@ -69,7 +69,7 @@ CREATE TABLE radgroupcheck (
   id int(11) unsigned NOT NULL auto_increment,
   GroupName varchar(64) NOT NULL default '',
   Attribute varchar(32)  NOT NULL default '',
-  op char(2) NOT NULL,
+  op char(2) NOT NULL DEFAULT '==',
   Value varchar(253)  NOT NULL default '',
   PRIMARY KEY  (id),
   KEY GroupName (GroupName(32))
@@ -83,7 +83,7 @@ CREATE TABLE radgroupreply (
   id int(11) unsigned NOT NULL auto_increment,
   GroupName varchar(64) NOT NULL default '',
   Attribute varchar(32)  NOT NULL default '',
-  op char(2) NOT NULL,
+  op char(2) NOT NULL DEFAULT '=',
   Value varchar(253)  NOT NULL default '',
   prio int unsigned NOT NULL default '0',
   PRIMARY KEY  (id),
@@ -98,7 +98,7 @@ CREATE TABLE radreply (
   id int(11) unsigned NOT NULL auto_increment,
   UserName varchar(64) NOT NULL default '',
   Attribute varchar(32) NOT NULL default '',
-  op char(2) NOT NULL,
+  op char(2) NOT NULL DEFAULT '=',
   Value varchar(253) NOT NULL default '',
   PRIMARY KEY  (id),
   KEY UserName (UserName(32))
