@@ -3,7 +3,7 @@
  *		accounting:     Functions to write radwtmp file.
  *		Also contains handler for "Group".
  *
- * Version:	$Id: rlm_unix.c,v 1.50 2002/10/28 20:14:03 aland Exp $
+ * Version:	$Id: rlm_unix.c,v 1.59 2003/07/07 19:11:45 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
  * Copyright 2000  Jeff Carneal <jeff@apex.net>
  * Copyright 2000  Alan Curry <pacman@world.std.com>
  */
-static const char rcsid[] = "$Id: rlm_unix.c,v 1.50 2002/10/28 20:14:03 aland Exp $";
+static const char rcsid[] = "$Id: rlm_unix.c,v 1.59 2003/07/07 19:11:45 aland Exp $";
 
 #include	"autoconf.h"
 #include	"libradius.h"
@@ -41,10 +41,6 @@ static const char rcsid[] = "$Id: rlm_unix.c,v 1.50 2002/10/28 20:14:03 aland Ex
 #  include	<shadow.h>
 #endif
 
-#if HAVE_CRYPT_H
-#  include <crypt.h>
-#endif
-
 #ifdef OSFC2
 #  include	<sys/security.h>
 #  include	<prot.h>
@@ -117,14 +113,15 @@ static int group_inst_explicit;
 
 #if HAVE_GETSPNAM
 #if defined(M_UNIX)
-	static inline const char *get_shadow_name(shadow_pwd_t *spwd) {
-		if (spwd == NULL) return NULL;
-		return (spwd->pw_name);
-	}
-	static inline const char *get_shadow_encrypted_pwd(shadow_pwd_t *spwd) {
-		if (spwd == NULL) return NULL;
-		return (spwd->pw_passwd);
-	}
+static inline const char *get_shadow_name(shadow_pwd_t *spwd) {
+	if (spwd == NULL) return NULL;
+	return (spwd->pw_name);
+}
+
+static inline const char *get_shadow_encrypted_pwd(shadow_pwd_t *spwd) {
+	if (spwd == NULL) return NULL;
+	return (spwd->pw_passwd);
+}
 #else /* M_UNIX */
 	static inline const char *get_shadow_name(shadow_pwd_t *spwd) {
 		if (spwd == NULL) return NULL;
@@ -212,7 +209,7 @@ static int groupcmp(void *instance, REQUEST *req, VALUE_PAIR *request, VALUE_PAI
 		return 1;
 	}
 
-	username = (char *)request->strvalue;
+	username = (char *)req->username->strvalue;
 
 	if (group_inst->cache_passwd &&
 	    (retval = H_groupcmp(group_inst->cache, check, username)) != -2)
@@ -265,7 +262,11 @@ static int unix_instantiate(CONF_SECTION *conf, void **instance)
 	/*
 	 *	Allocate room for the instance.
 	 */
-	inst = *instance = rad_malloc(sizeof(struct unix_instance));
+	inst = *instance = rad_malloc(sizeof(*inst));
+	if (!inst) {
+		return -1;
+	}
+	memset(inst, 0, sizeof(*inst));
 
 	/*
 	 *	Parse the configuration, failing if we can't do so.
@@ -284,10 +285,14 @@ static int unix_instantiate(CONF_SECTION *conf, void **instance)
                 {
 			radlog(L_ERR, "HASH:  unable to create user "
 				"hash table.  disable caching and run debugs");
-			free((char *) inst->passwd_file);
-			free((char *) inst->shadow_file);
-			free((char *) inst->group_file);
-			if (inst->radwtmp) free((char *) inst->radwtmp);
+			if (inst->passwd_file)
+				free((char *) inst->passwd_file);
+			if (inst->shadow_file)
+				free((char *) inst->shadow_file);
+			if (inst->group_file)
+				free((char *) inst->group_file);
+			if (inst->radwtmp)
+				free((char *) inst->radwtmp);
 			free(inst);
 			return -1;
 		}
@@ -325,10 +330,14 @@ static int unix_detach(void *instance)
 		group_inst = NULL;
 		group_inst_explicit = 0;
 	}
-	free((char *) inst->passwd_file);
-	free((char *) inst->shadow_file);
-	free((char *) inst->group_file);
-	if (inst->radwtmp) free((char *) inst->radwtmp);
+	if (inst->passwd_file)
+		free((char *) inst->passwd_file);
+	if (inst->shadow_file)
+		free((char *) inst->shadow_file);
+	if (inst->group_file)
+		free((char *) inst->group_file);
+	if (inst->radwtmp)
+		free((char *) inst->radwtmp);
 	if (inst->cache) {
 		unix_freepwcache(inst->cache);
 	}
@@ -356,8 +365,7 @@ static int unix_authenticate(void *instance, REQUEST *request)
 #define inst ((struct unix_instance *)instance)
 	char *name, *passwd;
 	struct passwd	*pwd;
-	char		*encpw;
-	char		*encrypted_pass;
+	const char	*encrypted_pass;
 	int		ret;
 #if HAVE_GETSPNAM
 	shadow_pwd_t	*spwd = NULL;
@@ -592,8 +600,7 @@ static int unix_authenticate(void *instance, REQUEST *request)
 	/*
 	 *	Check encrypted password.
 	 */
-	encpw = crypt(passwd, encrypted_pass);
-	if (strcmp(encpw, encrypted_pass)) {
+	if (lrad_crypt_check(passwd, encrypted_pass)) {
 		radlog(L_AUTH, "rlm_unix: [%s]: invalid password", name);
 		return RLM_MODULE_REJECT;
 	}
@@ -665,7 +672,7 @@ static int unix_accounting(void *instance, REQUEST *request)
 	 *	Which type is this.
 	 */
 	if ((vp = pairfind(request->packet->vps, PW_ACCT_STATUS_TYPE))==NULL) {
-		radlog(L_ERR, "Accounting: no Accounting-Status-Type record.");
+		radlog(L_ERR, "rlm_unix: no Accounting-Status-Type attribute in request.");
 		return RLM_MODULE_NOOP;
 	}
 	status = vp->lvalue;
@@ -709,7 +716,7 @@ static int unix_accounting(void *instance, REQUEST *request)
 			case PW_NAS_IP_ADDRESS:
 				nas_address = vp->lvalue;
 				break;
-			case PW_NAS_PORT_ID:
+			case PW_NAS_PORT:
 				nas_port = vp->lvalue;
 				port_seen = 1;
 				break;
@@ -724,7 +731,7 @@ static int unix_accounting(void *instance, REQUEST *request)
 
 	/*
 	 *	We don't store !root sessions, or sessions
-	 *	where we didn't see a PW_NAS_PORT_ID.
+	 *	where we didn't see a NAS-Port attribute.
 	 */
 	if (strncmp(ut.ut_name, "!root", sizeof(ut.ut_name)) == 0 || !port_seen)
 		return RLM_MODULE_NOOP;
@@ -759,7 +766,7 @@ static int unix_accounting(void *instance, REQUEST *request)
 #ifdef UT_HOSTSIZE
 	if (framed_address) {
 		ip_ntoa(buf, framed_address);
-		strncpy(ut.ut_host, buf, UT_HOSTSIZE);
+		strncpy(ut.ut_host, buf, sizeof(ut.ut_host));
 	}
 #endif
 #ifdef HAVE_UTMPX_H
